/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2021 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "table/8048.h"

struct _table_8048 table_8048[] =
{
  { "add",   0x68, 0xf8, 2, OP_A,     OP_RR,     1, 0 },
  { "add",   0x60, 0xfe, 2, OP_A,     OP_AT_R,   1, 0 },
  { "add",   0x03, 0xff, 2, OP_A,     OP_NUM,    2, 0 },
  { "addc",  0x78, 0xf8, 2, OP_A,     OP_RR,     1, 0 },
  { "addc",  0x70, 0xfe, 2, OP_A,     OP_AT_R,   1, 0 },
  { "addc",  0x13, 0xff, 2, OP_A,     OP_NUM,    2, 0 },
  { "anl",   0x58, 0xf8, 2, OP_A,     OP_RR,     1, 0 },
  { "anl",   0x50, 0xfe, 2, OP_A,     OP_AT_R,   1, 0 },
  { "anl",   0x53, 0xff, 2, OP_A,     OP_NUM,    2, 0 },
  { "anl",   0x98, 0xfc, 2, OP_P12,   OP_NUM,    2, FLAG_8048 },
  { "anld",  0x9c, 0xfc, 2, OP_PP,    OP_A,      2, 0 },
  { "call",  0x14, 0x1f, 1, OP_ADDR,  OP_NONE,   2, 0 },
  { "clr",   0x27, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "clr",   0x97, 0xff, 1, OP_C,     OP_NONE,   1, 0 },
  { "clr",   0x85, 0xff, 1, OP_F0,    OP_NONE,   1, 0 },
  { "clr",   0xa5, 0xff, 1, OP_F1,    OP_NONE,   1, 0 },
  { "cpl",   0x37, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "cpl",   0xa7, 0xff, 1, OP_C,     OP_NONE,   1, 0 },
  { "cpl",   0x95, 0xff, 1, OP_F0,    OP_NONE,   1, 0 },
  { "cpl",   0xb5, 0xff, 1, OP_F1,    OP_NONE,   1, 0 },
  { "da",    0x57, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "dec",   0x07, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "dec",   0xc8, 0xf8, 1, OP_RR,    OP_NONE,   1, 0 },
  { "dis",   0x15, 0xff, 1, OP_I,     OP_NONE,   1, 0 },
  { "dis",   0x35, 0xff, 1, OP_TCNTI, OP_NONE,   1, 0 },
  { "djnz",  0xe8, 0xff, 2, OP_RR,    OP_PADDR,  2, 0 },
  { "en",    0x05, 0xff, 1, OP_I,     OP_NONE,   1, 0 },
  { "en",    0x25, 0xff, 1, OP_TCNTI, OP_NONE,   1, 0 },
  { "en",    0xe5, 0xff, 1, OP_DMA,   OP_NONE,   1, FLAG_8041 },
  { "en",    0xf5, 0xff, 1, OP_FLAGS, OP_NONE,   1, FLAG_8041 },
  { "ent0",  0x75, 0xff, 1, OP_CLK,   OP_NONE,   1, FLAG_8048 },
  { "in",    0x08, 0xfc, 2, OP_A,     OP_P03,    2, 0 },
  { "in",    0x22, 0xff, 2, OP_A,     OP_DBB,    2, FLAG_8041 },
  { "inc",   0x17, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "inc",   0x18, 0xf8, 1, OP_RR,    OP_NONE,   1, 0 },
  { "inc",   0x10, 0xfe, 1, OP_AT_R,  OP_NONE,   1, 0 },
  { "ins",   0x08, 0xff, 2, OP_A,     OP_BUS,    2, FLAG_8048 },
  { "jb0",   0x12, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jb1",   0x32, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jb2",   0x52, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jb3",   0x72, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jb4",   0x92, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jb5",   0xb2, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jb6",   0xd2, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jb7",   0xf2, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jc",    0xf6, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jf0",   0xb6, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jf1",   0x76, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jmp",   0x04, 0x1f, 1, OP_ADDR,  OP_NONE,   2, 0 },
  { "jmpp",  0xb3, 0xff, 1, OP_AT_A,  OP_NONE,   2, 0 },
  { "jnc",   0xe6, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jni",   0x86, 0xff, 1, OP_PADDR, OP_NONE,   2, FLAG_8048 },
  { "jnibf", 0xd6, 0xff, 1, OP_PADDR, OP_NONE,   2, FLAG_8041 },
  { "jnt0",  0x26, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jnt1",  0x46, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jnz",   0x96, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jobf",  0x86, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jtf",   0x16, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jt0",   0x36, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jt1",   0x56, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "jz",    0xc6, 0xff, 1, OP_PADDR, OP_NONE,   2, 0 },
  { "mov",   0x23, 0xff, 2, OP_A,     OP_NUM,    2, 0 },
  { "mov",   0xc7, 0xff, 2, OP_A,     OP_PSW,    1, 0 },
  { "mov",   0xf8, 0xf8, 2, OP_A,     OP_RR,     1, 0 },
  { "mov",   0xf0, 0xfe, 2, OP_A,     OP_AT_R,   1, 0 },
  { "mov",   0x42, 0xff, 2, OP_A,     OP_T,      1, 0 },
  { "mov",   0xd7, 0xff, 2, OP_PSW,   OP_A,      1, 0 },
  { "mov",   0xa8, 0xf8, 2, OP_RR,    OP_A,      1, 0 },
  { "mov",   0xb8, 0xf8, 2, OP_RR,    OP_NUM,    2, 0 },
  { "mov",   0xa0, 0xfe, 2, OP_AT_R,  OP_A,      1, 0 },
  { "mov",   0xb0, 0xfe, 2, OP_AT_R,  OP_NUM,    2, 0 },
  { "mov",   0x62, 0xff, 2, OP_T,     OP_A,      1, 0 },
  { "mov",   0x90, 0xff, 2, OP_STS,   OP_A,      1, FLAG_8041 },
  { "movd",  0x0c, 0xfc, 2, OP_A,     OP_PP,     2, 0 },
  { "movd",  0x3c, 0xfc, 2, OP_PP,    OP_A,      2, 0 },
  { "movp",  0xa3, 0xff, 2, OP_A,     OP_AT_A,   1, 0 },
  { "movp3", 0xe3, 0xff, 2, OP_A,     OP_AT_A,   2, 0 },
  { "movx",  0x80, 0xfe, 2, OP_A,     OP_AT_R,   2, FLAG_8048 },
  { "movx",  0x90, 0xfe, 2, OP_AT_R,  OP_A,      2, FLAG_8048 },
  { "nop",   0x00, 0xff, 0, OP_NONE,  OP_NONE,   1, 0 },
  { "orl",   0x48, 0xf8, 2, OP_A,     OP_RR,     1, 0 },
  { "orl",   0x40, 0xfe, 2, OP_A,     OP_AT_R,   1, 0 },
  { "orl",   0x43, 0xff, 2, OP_A,     OP_NUM,    2, 0 },
  { "orl",   0x88, 0xff, 2, OP_P12,   OP_NUM,    2, FLAG_8048 },
  { "orld",  0x8c, 0xfc, 2, OP_PP,    OP_A,      2, 0 },
  { "out",   0x02, 0xff, 2, OP_DBB,   OP_A,      2, FLAG_8041 },
  { "outl",  0x02, 0xff, 2, OP_BUS,   OP_A,      2, FLAG_8048 },
  { "outl",  0x90, 0xfc, 2, OP_P0,    OP_A,      2, 0 },
  { "ret",   0x83, 0xff, 0, OP_NONE,  OP_NONE,   2, 0 },
  { "retr",  0x93, 0xff, 0, OP_NONE,  OP_NONE,   2, 0 },
  { "rl",    0xe7, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "rlc",   0xf7, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "rr",    0x77, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "rrc",   0x67, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "sel",   0xe5, 0xff, 1, OP_MB0,   OP_NONE,   1, FLAG_8048 },
  { "sel",   0xf5, 0xff, 1, OP_MB1,   OP_NONE,   1, FLAG_8048 },
  { "sel",   0xc5, 0xff, 1, OP_RB0,   OP_NONE,   1, 0 },
  { "sel",   0xd5, 0xff, 1, OP_RB1,   OP_NONE,   1, 0 },
  { "stop",  0x65, 0xff, 1, OP_TCNT,  OP_NONE,   1, 0 },
  { "strt",  0x45, 0xff, 1, OP_CNT,   OP_NONE,   1, 0 },
  { "strt",  0x55, 0xff, 1, OP_T,     OP_NONE,   1, 0 },
  { "swap",  0x47, 0xff, 1, OP_A,     OP_NONE,   1, 0 },
  { "xch",   0x28, 0xf8, 2, OP_A,     OP_RR,     1, 0 },
  { "xch",   0x20, 0xfe, 2, OP_A,     OP_AT_R,   1, 0 },
  { "xchd",  0x30, 0xfe, 2, OP_A,     OP_AT_R,   1, 0 },
  { "xrl",   0xd8, 0xf8, 2, OP_A,     OP_RR,     1, 0 },
  { "xrl",   0xd0, 0xfe, 2, OP_A,     OP_AT_R,   1, 0 },
  { "xrl",   0xd3, 0xff, 2, OP_A,     OP_NUM,    2, 0 },
  { NULL,    0x00, 0x00, 0, OP_NONE,  OP_NONE,   0, 0 },
};

