/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2019 by Michael Kohn
 *
 */

#include "table/avr8.h"

struct _table_avr8 table_avr8[] =
{
  { "sec", 0x9408, 0xffff, OP_NONE, 1, 1, AVR8_SEC },
  { "clc", 0x9488, 0xffff, OP_NONE, 1, 1, AVR8_CLC },
  { "sen", 0x9428, 0xffff, OP_NONE, 1, 1, AVR8_SEN },
  { "cln", 0x94a8, 0xffff, OP_NONE, 1, 1, AVR8_CLN },
  { "sez", 0x9418, 0xffff, OP_NONE, 1, 1, AVR8_SEZ },
  { "clz", 0x9498, 0xffff, OP_NONE, 1, 1, AVR8_CLZ },
  { "sei", 0x9478, 0xffff, OP_NONE, 1, 1, AVR8_SEI },
  { "cli", 0x94f8, 0xffff, OP_NONE, 1, 1, AVR8_CLI },
  { "ses", 0x9448, 0xffff, OP_NONE, 1, 1, AVR8_SES },
  { "cls", 0x94c8, 0xffff, OP_NONE, 1, 1, AVR8_CLS },
  { "sev", 0x9438, 0xffff, OP_NONE, 1, 1, AVR8_SEV },
  { "clv", 0x94b8, 0xffff, OP_NONE, 1, 1, AVR8_CLV },
  { "set", 0x9468, 0xffff, OP_NONE, 1, 1, AVR8_SET },
  { "clt", 0x94e8, 0xffff, OP_NONE, 1, 1, AVR8_CLT },
  { "seh", 0x9458, 0xffff, OP_NONE, 1, 1, AVR8_SEH },
  { "clh", 0x94d8, 0xffff, OP_NONE, 1, 1, AVR8_CLH },
  { "break", 0x9598, 0xffff, OP_NONE, 1, 1, AVR8_BREAK },
  { "nop", 0x0000, 0xffff, OP_NONE, 1, 1, AVR8_NOP },
  { "sleep", 0x9588, 0xffff, OP_NONE, 1, 1, AVR8_SLEEP },
  { "wdr", 0x95a8, 0xffff, OP_NONE, 1, 1, AVR8_WDR },
  { "ijmp", 0x9409, 0xffff, OP_NONE, 2, 2, AVR8_IJMP },
  { "eijmp", 0x9419, 0xffff, OP_NONE, 2, 2, AVR8_EIJMP },
  { "icall", 0x9509, 0xffff, OP_NONE, 3, 3, AVR8_ICALL },
  { "eicall", 0x9519, 0xffff, OP_NONE, 4, 4, AVR8_EICALL },
  { "ret", 0x9508, 0xffff, OP_NONE, 4, 4, AVR8_RET },
  { "reti", 0x9518, 0xffff, OP_NONE, 4, 4, AVR8_RETI },
  { "lpm", 0x95c8, 0xffff, OP_NONE, 3, 3, AVR8_LPM },
  { "elpm", 0x95d8, 0xffff, OP_NONE, 3, 3, AVR8_ELPM },
  { "spm", 0x95e8, 0xffff, OP_NONE, -1, -1, AVR8_SPM },
  { "breq", 0xf001, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BREQ },
  { "brne", 0xf401, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRNE },
  { "brcs", 0xf000, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRCS },
  { "brcc", 0xf400, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRCC },
  { "brsh", 0xf400, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRSH },
  { "brlo", 0xf000, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRLO },
  { "brmi", 0xf002, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRMI },
  { "brpl", 0xf402, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRPL },
  { "brge", 0xf404, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRGE },
  { "brlt", 0xf004, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRLT },
  { "brhs", 0xf005, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRHS },
  { "brhc", 0xf405, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRHC },
  { "brts", 0xf006, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRTS },
  { "brtc", 0xf406, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRTC },
  { "brvs", 0xf003, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRVS },
  { "brvc", 0xf403, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRVC },
  { "brie", 0xf007, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRIE },
  { "brid", 0xf407, 0xfc07, OP_BRANCH_K, 1, 2, AVR8_BRID },
  { "brbs", 0xf000, 0xfc00, OP_BRANCH_S_K, 1, 2, AVR8_BRBS },
  { "brbc", 0xf400, 0xfc00, OP_BRANCH_S_K, 1, 2, AVR8_BRBC },
  { "adc", 0x1c00, 0xfc00, OP_TWO_REG, 1, 1, AVR8_ADC },
  { "add", 0x0c00, 0xfc00, OP_TWO_REG, 1, 1, AVR8_ADD },
  { "and", 0x2000, 0xfc00, OP_TWO_REG, 1, 1, AVR8_AND },
  { "cp", 0x1400, 0xfc00, OP_TWO_REG, 1, 1, AVR8_CP },
  { "cpc", 0x0400, 0xfc00, OP_TWO_REG, 1, 1, AVR8_CPC },
  { "cpse", 0x1000, 0xfc00, OP_TWO_REG, 1, 3, AVR8_CPSE },
  { "eor", 0x2400, 0xfc00, OP_TWO_REG, 1, 1, AVR8_EOR },
  { "mov", 0x2c00, 0xfc00, OP_TWO_REG, 1, 1, AVR8_MOV },
  { "mul", 0x9c00, 0xfc00, OP_TWO_REG, 2, 2, AVR8_MUL },
  { "or", 0x2800, 0xfc00, OP_TWO_REG, 1, 1, AVR8_OR },
  { "sbc", 0x0800, 0xfc00, OP_TWO_REG, 1, 1, AVR8_SBC },
  { "sub", 0x1800, 0xfc00, OP_TWO_REG, 1, 1, AVR8_SUB },
  //{ "tst", 0x2000, 0xfc00, OP_RD, 1, 1, AVR8_TST },   and
  //{ "clr", 0x2400, 0xfc00, OP_RD, 1, 1, AVR8_CLR },   eor
  //{ "lsl", 0x0c00, 0xfc00, OP_RD, 1, 1, AVR8_LSL },
  //{ "rol", 0x1c00, 0xfc00, OP_RD, 1, 1, AVR8_ROL },
  { "andi", 0x7000, 0xf000, OP_REG_IMM, 1, 1, AVR8_ANDI },
  { "cpi", 0x3000, 0xf000, OP_REG_IMM, 1, 1, AVR8_CPI },
  { "ldi", 0xe000, 0xf000, OP_REG_IMM, 1, 1, AVR8_LDI },
  { "ori", 0x6000, 0xf000, OP_REG_IMM, 1, 1, AVR8_ORI },
  { "sbci", 0x4000, 0xf000, OP_REG_IMM, 1, 1, AVR8_SBCI },
  { "sbr", 0x6000, 0xf000, OP_REG_IMM, 1, 1, AVR8_SBR },
  { "subi", 0x5000, 0xf000, OP_REG_IMM, 1, 1, AVR8_SUBI },
  //{ "cbr", 0x0000, 0x0000, OP_REG_IMM, 1, 1, AVR8_CBR },
  { "asr", 0x9405, 0xfe0f, OP_ONE_REG, 1, 1, AVR8_ASR },
  { "com", 0x9400, 0xfe0f, OP_ONE_REG, 1, 1, AVR8_COM },
  { "dec", 0x940a, 0xfe0f, OP_ONE_REG, 1, 1, AVR8_DEC },
  { "inc", 0x9403, 0xfe0f, OP_ONE_REG, 1, 1, AVR8_INC },
  { "lsr", 0x9406, 0xfe0f, OP_ONE_REG, 1, 1, AVR8_LSR },
  { "neg", 0x9401, 0xfe0f, OP_ONE_REG, 1, 1, AVR8_NEG },
  { "pop", 0x900f, 0xfe0f, OP_ONE_REG, 2, 2, AVR8_POP },
  { "push", 0x920f, 0xfe0f, OP_ONE_REG, 2, 2, AVR8_PUSH },
  { "ror", 0x9407, 0xfe0f, OP_ONE_REG, 1, 1, AVR8_ROR },
  { "swap", 0x9402, 0xfe0f, OP_ONE_REG, 1, 1, AVR8_SWAP },
  { "bld", 0xf800, 0xfe08, OP_REG_BIT, 1, 1, AVR8_BLD },
  { "bst", 0xfa00, 0xfe08, OP_REG_BIT, 1, 1, AVR8_BST },
  { "sbrc", 0xfc00, 0xfe08, OP_REG_BIT, 1, 3, AVR8_SBRC },
  { "sbrs", 0xfe00, 0xfe08, OP_REG_BIT, 1, 3, AVR8_SBRS },
  { "adiw", 0x9600, 0xff00, OP_REG_IMM_WORD, 2, 2, AVR8_ADIW },
  { "sbiw", 0x9700, 0xff00, OP_REG_IMM_WORD, 2, 2, AVR8_SBIW },
  { "cbi", 0x9800, 0xff00, OP_IOREG_BIT, 2, 2, AVR8_CBI },
  { "sbi", 0x9a00, 0xff00, OP_IOREG_BIT, 2, 2, AVR8_SBI },
  { "sbic", 0x9900, 0xff00, OP_IOREG_BIT, 1, 3, AVR8_SBIC },
  { "sbis", 0x9b00, 0xff00, OP_IOREG_BIT, 1, 3, AVR8_SBIS },
  { "bset", 0x9408, 0xff8f, OP_SREG_BIT, 1, 1, AVR8_BSET },
  { "bclr", 0x9488, 0xff8f, OP_SREG_BIT, 1, 1, AVR8_BCLR },
  { "ser", 0xef0f, 0xff0f, OP_REG_4, 1, 1, AVR8_SER },
  { "in", 0xb000, 0xf800, OP_IN, 1, 1, AVR8_IN },
  { "out", 0xb800, 0xf800, OP_OUT, 1, 1, AVR8_OUT },
  { "movw", 0x0100, 0xff00, OP_MOVW, 1, 1, AVR8_MOVW },
  { "rjmp", 0xc000, 0xf000, OP_RELATIVE, 2, 2, AVR8_RJMP },
  { "rcall", 0xd000, 0xf000, OP_RELATIVE, 3, 3, AVR8_RCALL },
  { "call", 0x940e, 0xfe0e, OP_JUMP, 4, 4, AVR8_CALL },
  { "jmp", 0x940c, 0xfe0e, OP_JUMP, 3, 3, AVR8_JMP },
  { "ld", 0x900c, 0xfe0f, OP_REG_X, 2, 2, AVR8_LD },
  { "ld", 0x8008, 0xfe0f, OP_REG_Y, 2, 2, AVR8_LD },
  { "ld", 0x8000, 0xfe0f, OP_REG_Z, 2, 2, AVR8_LD },
  { "ld", 0x900d, 0xfe0f, OP_REG_X_PLUS, 2, 2, AVR8_LD },
  { "ld", 0x9009, 0xfe0f, OP_REG_Y_PLUS, 2, 2, AVR8_LD },
  { "ld", 0x9001, 0xfe0f, OP_REG_Z_PLUS, 2, 2, AVR8_LD },
  { "ld", 0x900e, 0xfe0f, OP_REG_MINUS_X, 2, 2, AVR8_LD },
  { "ld", 0x900a, 0xfe0f, OP_REG_MINUS_Y, 2, 2, AVR8_LD },
  { "ld", 0x9002, 0xfe0f, OP_REG_MINUS_Z, 2, 2, AVR8_LD },
  { "st", 0x920c, 0xfe0f, OP_X_REG, 2, 2, AVR8_ST },
  { "st", 0x8208, 0xfe0f, OP_Y_REG, 2, 2, AVR8_ST },
  { "st", 0x8200, 0xfe0f, OP_Z_REG, 2, 2, AVR8_ST },
  { "st", 0x920d, 0xfe0f, OP_X_PLUS_REG, 2, 2, AVR8_ST },
  { "st", 0x9209, 0xfe0f, OP_Y_PLUS_REG, 2, 2, AVR8_ST },
  { "st", 0x9201, 0xfe0f, OP_Z_PLUS_REG, 2, 2, AVR8_ST },
  { "st", 0x920e, 0xfe0f, OP_MINUS_X_REG, 2, 2, AVR8_ST },
  { "st", 0x920a, 0xfe0f, OP_MINUS_Y_REG, 2, 2, AVR8_ST },
  { "st", 0x9202, 0xfe0f, OP_MINUS_Z_REG, 2, 2, AVR8_ST },
  { "lpm", 0x9004, 0xfe0f, OP_REG_Z, 3, 3, AVR8_LPM },
  { "lpm", 0x9005, 0xfe0f, OP_REG_Z_PLUS, 3, 3, AVR8_LPM },
  { "elpm", 0x9006, 0xfe0f, OP_REG_Z, 3, 3, AVR8_ELPM },
  { "elpm", 0x9007, 0xfe0f, OP_REG_Z_PLUS, 3, 3, AVR8_ELPM },
  { "fmuls", 0x0380, 0xff88, OP_FMUL, 2, 2, AVR8_FMULS },
  { "fmul", 0x0308, 0xff88, OP_FMUL, 2, 2, AVR8_FMUL },
  { "fmulsu", 0x0388, 0xff88, OP_FMUL, 2, 2, AVR8_FMULSU },
  { "mulsu", 0x0300, 0xff88, OP_FMUL, 2, 2, AVR8_MULSU },
  { "muls", 0x0200, 0xff00, OP_MULS, 2, 2, AVR8_MULS },
  { "des", 0x940b, 0xff0f, OP_DATA4, 1, 2, AVR8_DES },
  { "lds", 0x9000, 0xfe0f, OP_REG_SRAM, 3, 3, AVR8_LDS },
  { "sts", 0x9200, 0xfe0f, OP_SRAM_REG, 3, 3, AVR8_STS },
  { "ldd", 0x8008, 0xd208, OP_REG_Y_PLUS_Q, 2, 2, AVR8_LDD },
  { "ldd", 0x8000, 0xd208, OP_REG_Z_PLUS_Q, 2, 2, AVR8_LDD },
  { "std", 0x8208, 0xd208, OP_Y_PLUS_Q_REG, 2, 2, AVR8_STD },
  { "std", 0x8200, 0xd208, OP_Z_PLUS_Q_REG, 2, 2, AVR8_STD },
  { "spm", 0x95e8, 0xffff, OP_SPM_Z_PLUS, -1, -1, AVR8_SPM },  // all devices
  { "spm", 0x94c8, 0xffff, OP_NONE, -1, -1, AVR8_SPM },        // #2?
  { "spm", 0x95f8, 0xffff, OP_SPM_Z_PLUS, -1, -1, AVR8_SPM },  // #2?
  { NULL, 0, 0, 0, 0, 0 }
};

