/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2019 by Michael Kohn
 *
 */

#include <stdlib.h>

#include "table/dotnet.h"

struct _table_dotnet table_dotnet[] =
{
  { "add",            0x58, DOTNET_OP_NONE },
  { "add.ovf",        0xd6, DOTNET_OP_NONE },
  { "add.ovf.un",     0xd7, DOTNET_OP_NONE },
  { "and",            0x5f, DOTNET_OP_NONE },
  { "beq",            0x3b, DOTNET_OP_BRANCH32 },
  { "beq.s",          0x2e, DOTNET_OP_BRANCH8 },
  { "bge",            0x3c, DOTNET_OP_BRANCH32 },
  { "bge.s",          0x2f, DOTNET_OP_BRANCH8 },
  { "bge.un",         0x41, DOTNET_OP_BRANCH32 },
  { "bge.un.s",       0x34, DOTNET_OP_BRANCH8 },
  { "bgt",            0x3d, DOTNET_OP_BRANCH32 },
  { "bgt.s",          0x30, DOTNET_OP_BRANCH8 },
  { "bgt.un",         0x42, DOTNET_OP_BRANCH32 },
  { "bgt.un.s",       0x35, DOTNET_OP_BRANCH8 },
  { "ble",            0x3e, DOTNET_OP_BRANCH32 },
  { "ble.s",          0x31, DOTNET_OP_BRANCH8 },
  { "ble.un",         0x43, DOTNET_OP_BRANCH32 },
  { "ble.un.s",       0x36, DOTNET_OP_BRANCH8 },
  { "blt",            0x3f, DOTNET_OP_BRANCH32 },
  { "blt.s",          0x32, DOTNET_OP_BRANCH8 },
  { "blt.un",         0x44, DOTNET_OP_BRANCH32 },
  { "blt.un.s",       0x37, DOTNET_OP_BRANCH8 },
  { "bne.un",         0x40, DOTNET_OP_BRANCH32 },
  { "bne.un.s",       0x33, DOTNET_OP_BRANCH8 },
  { "box",            0x8c, DOTNET_OP_TYPE_TOK },
  { "br",             0x38, DOTNET_OP_BRANCH32 },
  { "br.s",           0x2b, DOTNET_OP_BRANCH8 },
  { "break",          0x01, DOTNET_OP_NONE },
  { "brfalse",        0x39, DOTNET_OP_BRANCH32 },
  { "brfalse.s",      0x2c, DOTNET_OP_BRANCH8 },
  { "brinst",         0x3a, DOTNET_OP_BRANCH32 },
  { "brinst.s",       0x2d, DOTNET_OP_BRANCH8 },
  { "brnull",         0x39, DOTNET_OP_BRANCH32 },
  { "brnull.s",       0x2c, DOTNET_OP_BRANCH8 },
  { "brtrue",         0x3a, DOTNET_OP_BRANCH32 },
  { "brtrue.s",       0x2d, DOTNET_OP_BRANCH8 },
  { "brzero",         0x39, DOTNET_OP_BRANCH32 },
  { "brzero.s",       0x2c, DOTNET_OP_BRANCH8 },
  { "call",           0x28, DOTNET_OP_METHOD },
  { "calli",          0x29, DOTNET_OP_NONE },
  { "callvirt",       0x6f, DOTNET_OP_CALLSITEDESCR },
  { "castclass",      0x74, DOTNET_OP_CLASS },
  { "ckfinite",       0xc3, DOTNET_OP_NONE },
  { "conv.i",         0xd3, DOTNET_OP_NONE },
  { "conv.i1",        0x67, DOTNET_OP_NONE },
  { "conv.i2",        0x68, DOTNET_OP_NONE },
  { "conv.i4",        0x69, DOTNET_OP_NONE },
  { "conv.i8",        0x6a, DOTNET_OP_NONE },
  { "conv.ovf.i",     0xd4, DOTNET_OP_NONE },
  { "conv.ovf.i.un",  0x8a, DOTNET_OP_NONE },
  { "conv.ovf.i1",    0xb3, DOTNET_OP_NONE },
  { "conv.ovf.i1.un", 0x82, DOTNET_OP_NONE },
  { "conv.ovf.i2",    0xb5, DOTNET_OP_NONE },
  { "conv.ovf.i2.un", 0x83, DOTNET_OP_NONE },
  { "conv.ovf.i4",    0xb7, DOTNET_OP_NONE },
  { "conv.ovf.i4.un", 0x84, DOTNET_OP_NONE },
  { "conv.ovf.i8",    0xb9, DOTNET_OP_NONE },
  { "conv.ovf.i8.un", 0x85, DOTNET_OP_NONE },
  { "conv.ovf.u",     0xd5, DOTNET_OP_NONE },
  { "conv.ovf.u.un",  0x8b, DOTNET_OP_NONE },
  { "conv.ovf.u1",    0xb4, DOTNET_OP_NONE },
  { "conv.ovf.u1.un", 0x86, DOTNET_OP_NONE },
  { "conv.ovf.u2",    0xb6, DOTNET_OP_NONE },
  { "conv.ovf.u2.un", 0x87, DOTNET_OP_NONE },
  { "conv.ovf.u4",    0xb8, DOTNET_OP_NONE },
  { "conv.ovf.u4.un", 0x88, DOTNET_OP_NONE },
  { "conv.ovf.u8",    0xba, DOTNET_OP_NONE },
  { "conv.ovf.u8.un", 0x89, DOTNET_OP_NONE },
  { "conv.r.un",      0x76, DOTNET_OP_NONE },
  { "conv.r4",        0x6b, DOTNET_OP_NONE },
  { "conv.r8",        0x6c, DOTNET_OP_NONE },
  { "conv.u",         0xe0, DOTNET_OP_NONE },
  { "conv.u1",        0xd2, DOTNET_OP_NONE },
  { "conv.u2",        0xd1, DOTNET_OP_NONE },
  { "conv.u4",        0x6d, DOTNET_OP_NONE },
  { "conv.u8",        0x6e, DOTNET_OP_NONE },
  { "cpobj",          0x70, DOTNET_OP_TYPE_TOK },
  { "div",            0x5b, DOTNET_OP_NONE },
  { "div.un",         0x5c, DOTNET_OP_NONE },
  { "dup",            0x25, DOTNET_OP_NONE },
  { "endfault",       0xdc, DOTNET_OP_NONE },
  { "endfinally",     0xdc, DOTNET_OP_NONE },
  { "isinst",         0x75, DOTNET_OP_CLASS },
  { "jmp",            0x27, DOTNET_OP_METHOD },
  { "ldarg.0",        0x02, DOTNET_OP_NONE },
  { "ldarg.1",        0x03, DOTNET_OP_NONE },
  { "ldarg.2",        0x04, DOTNET_OP_NONE },
  { "ldarg.3",        0x05, DOTNET_OP_NONE },
  { "ldarg.s",        0x0e, DOTNET_OP_UINT8 },
  { "ldarga.s",       0x0f, DOTNET_OP_UINT8 },
  { "ldc.i4",         0x20, DOTNET_OP_INT32 },
  { "ldc.i4.0",       0x16, DOTNET_OP_NONE },
  { "ldc.i4.1",       0x17, DOTNET_OP_NONE },
  { "ldc.i4.2",       0x18, DOTNET_OP_NONE },
  { "ldc.i4.3",       0x19, DOTNET_OP_NONE },
  { "ldc.i4.4",       0x1a, DOTNET_OP_NONE },
  { "ldc.i4.5",       0x1b, DOTNET_OP_NONE },
  { "ldc.i4.6",       0x1c, DOTNET_OP_NONE },
  { "ldc.i4.7",       0x1d, DOTNET_OP_NONE },
  { "ldc.i4.8",       0x1e, DOTNET_OP_NONE },
  { "ldc.i4.m1",      0x15, DOTNET_OP_NONE },
  { "ldc.i4.m1",      0x15, DOTNET_OP_NONE },
  { "ldc.i4.s",       0x1f, DOTNET_OP_INT8 },
  { "ldc.i8",         0x21, DOTNET_OP_INT64 },
  { "ldc.r4",         0x22, DOTNET_OP_FLOAT32 },
  { "ldc.r8",         0x23, DOTNET_OP_FLOAT64 },
  { "ldelem",         0xa3, DOTNET_OP_TYPE_TOK },
  { "ldelem.i",       0x97, DOTNET_OP_NONE },
  { "ldelem.i1",      0x90, DOTNET_OP_NONE },
  { "ldelem.i2",      0x92, DOTNET_OP_NONE },
  { "ldelem.i4",      0x94, DOTNET_OP_NONE },
  { "ldelem.i8",      0x96, DOTNET_OP_NONE },
  { "ldelem.r4",      0x98, DOTNET_OP_NONE },
  { "ldelem.r8",      0x99, DOTNET_OP_NONE },
  { "ldelem.ref",     0x9a, DOTNET_OP_NONE },
  { "ldelem.u1",      0x91, DOTNET_OP_NONE },
  { "ldelem.u2",      0x93, DOTNET_OP_NONE },
  { "ldelem.u4",      0x95, DOTNET_OP_NONE },
  { "ldelem.u8",      0x96, DOTNET_OP_NONE },
  { "ldelema",        0x8f, DOTNET_OP_CLASS },
  { "ldfld",          0x7b, DOTNET_OP_FIELD },
  { "ldflda",         0x7c, DOTNET_OP_FIELD },
  { "ldind.i",        0x4d, DOTNET_OP_NONE },
  { "ldind.i1",       0x46, DOTNET_OP_NONE },
  { "ldind.i2",       0x48, DOTNET_OP_NONE },
  { "ldind.i4",       0x4a, DOTNET_OP_NONE },
  { "ldind.i8",       0x4c, DOTNET_OP_NONE },
  { "ldind.r4",       0x4e, DOTNET_OP_NONE },
  { "ldind.r8",       0x4f, DOTNET_OP_NONE },
  { "ldind.ref",      0x50, DOTNET_OP_NONE },
  { "ldind.u1",       0x47, DOTNET_OP_NONE },
  { "ldind.u2",       0x49, DOTNET_OP_NONE },
  { "ldind.u4",       0x4b, DOTNET_OP_NONE },
  { "ldind.u8",       0x4c, DOTNET_OP_NONE },
  { "ldlen",          0x8e, DOTNET_OP_NONE },
  { "ldloc.0",        0x06, DOTNET_OP_NONE },
  { "ldloc.1",        0x07, DOTNET_OP_NONE },
  { "ldloc.2",        0x08, DOTNET_OP_NONE },
  { "ldloc.3",        0x09, DOTNET_OP_NONE },
  { "ldloc.s",        0x11, DOTNET_OP_UINT8 },
  { "ldloca.s",       0x12, DOTNET_OP_UINT8 },
  { "ldnull",         0x14, DOTNET_OP_NONE },
  { "ldobj",          0x71, DOTNET_OP_TYPE_TOK },
  { "ldsfld",         0x7e, DOTNET_OP_FIELD },
  { "ldsflda",        0x7f, DOTNET_OP_FIELD },
  { "ldstr",          0x72, DOTNET_OP_STRING },
  { "ldtoken",        0xd0, DOTNET_OP_TOKEN },
  { "leave",          0xdd, DOTNET_OP_INT32 },
  { "leave.s",        0xde, DOTNET_OP_INT8 },
  { "mkrefany",       0xc6, DOTNET_OP_CLASS },
  { "mul",            0x5a, DOTNET_OP_NONE },
  { "mul.ovf",        0xd8, DOTNET_OP_NONE },
  { "mul.ovf.un",     0xd9, DOTNET_OP_NONE },
  { "neg",            0x65, DOTNET_OP_NONE },
  { "newarr",         0x8d, DOTNET_OP_ETYPE },
  { "newobj",         0x73, DOTNET_OP_CTOR },
  { "nop",            0x00, DOTNET_OP_NONE },
  { "not",            0x66, DOTNET_OP_NONE },
  { "or",             0x60, DOTNET_OP_NONE },
  { "pop",            0x26, DOTNET_OP_NONE },
  { "refanyval",      0xc2, DOTNET_OP_TYPE },
  { "rem",            0x5d, DOTNET_OP_NONE },
  { "rem.un",         0x5e, DOTNET_OP_NONE },
  { "ret",            0x2a, DOTNET_OP_NONE },
  { "shl",            0x62, DOTNET_OP_NONE },
  { "shr",            0x63, DOTNET_OP_NONE },
  { "shr.un",         0x64, DOTNET_OP_NONE },
  { "starg.s",        0x10, DOTNET_OP_UINT8 },
  { "stelem",         0xa4, DOTNET_OP_TYPE_TOK },
  { "stelem.i",       0x9b, DOTNET_OP_NONE },
  { "stelem.i1",      0x9c, DOTNET_OP_NONE },
  { "stelem.i2",      0x9d, DOTNET_OP_NONE },
  { "stelem.i4",      0x9e, DOTNET_OP_NONE },
  { "stelem.i8",      0x9f, DOTNET_OP_NONE },
  { "stelem.r4",      0xa0, DOTNET_OP_NONE },
  { "stelem.r8",      0xa1, DOTNET_OP_NONE },
  { "stelem.ref",     0xa2, DOTNET_OP_NONE },
  { "stfld",          0x7d, DOTNET_OP_FIELD },
  { "stind.i",        0xdf, DOTNET_OP_NONE },
  { "stind.i1",       0x52, DOTNET_OP_NONE },
  { "stind.i2",       0x53, DOTNET_OP_NONE },
  { "stind.i4",       0x54, DOTNET_OP_NONE },
  { "stind.i8",       0x55, DOTNET_OP_NONE },
  { "stind.r4",       0x56, DOTNET_OP_NONE },
  { "stind.r8",       0x57, DOTNET_OP_NONE },
  { "stind.ref",      0x51, DOTNET_OP_NONE },
  { "stloc.0",        0x0a, DOTNET_OP_UINT16 },
  { "stloc.1",        0x0b, DOTNET_OP_NONE },
  { "stloc.2",        0x0c, DOTNET_OP_NONE },
  { "stloc.3",        0x0d, DOTNET_OP_NONE },
  { "stloc.s",        0x13, DOTNET_OP_UINT8 },
  { "stobj",          0x81, DOTNET_OP_TYPE_TOK },
  { "stsfld",         0x80, DOTNET_OP_FIELD },
  { "sub",            0x59, DOTNET_OP_NONE },
  { "sub.ovf",        0xda, DOTNET_OP_NONE },
  { "sub.ovf.un",     0xdb, DOTNET_OP_NONE },
  { "switch",         0x45, DOTNET_OP_SWITCH },
  { "throw",          0x7a, DOTNET_OP_NONE },
  { "unbox",          0x79, DOTNET_OP_VALUE_TYPE },
  { "unbox.any",      0xa5, DOTNET_OP_TYPE_TOK },
  { "xor",            0x61, DOTNET_OP_NONE },
  { NULL,             0x00, DOTNET_OP_ILLEGAL },
};

struct _table_dotnet table_dotnet_fe[] =
{
  { "arglist",        0x00, DOTNET_OP_NONE },
  { "ceq",            0x01, DOTNET_OP_NONE },
  { "cgt",            0x02, DOTNET_OP_NONE },
  { "cgt.un",         0x03, DOTNET_OP_NONE },
  { "clt",            0x04, DOTNET_OP_NONE },
  { "clt.un",         0x05, DOTNET_OP_NONE },
  { "constrained.",   0x16, DOTNET_OP_THIS_TYPE },
  { "cpblk",          0x17, DOTNET_OP_NONE },
  { "endfilter",      0x11, DOTNET_OP_NONE },
  { "initblk",        0x18, DOTNET_OP_NONE },
  { "initobj",        0x15, DOTNET_OP_TYPE_TOK },
  { "ldarg",          0x09, DOTNET_OP_UINT16 },
  { "ldarga",         0x0a, DOTNET_OP_UINT16 },
  { "ldftn",          0x06, DOTNET_OP_METHOD },
  { "ldloc",          0x0c, DOTNET_OP_UINT16 },
  { "ldloca",         0x0d, DOTNET_OP_UINT16 },
  { "ldvirtftn",      0x07, DOTNET_OP_METHOD },
  { "localloc",       0x0f, DOTNET_OP_NONE },
  { "no.",            0x19, DOTNET_OP_PREFIX_CHECK },
  { "readonly.",      0x1e, DOTNET_OP_NONE },
  { "refanytype",     0x1d, DOTNET_OP_NONE },
  { "rethrow",        0x1a, DOTNET_OP_NONE },
  { "sizeof",         0x1c, DOTNET_OP_TYPE_TOK },
  { "starg",          0x0b, DOTNET_OP_UINT16 },
  { "stloc",          0x0e, DOTNET_OP_NONE },
  { "tail.",          0x14, DOTNET_OP_NONE },
  { "unaligned.",     0x12, DOTNET_OP_PREFIX_ALIGNED },
  { "volatile.",      0x13, DOTNET_OP_NONE },
  { NULL,             0x00, DOTNET_OP_ILLEGAL },
};

