/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2019 by Michael Kohn
 *
 */

#include <stdlib.h>

#include "table/java.h"

struct _table_java table_java[] =
{
  { "nop",             1,  0,  JAVA_OP_NONE },  // nop (0x00)
  { "aconst_null",     1,  0,  JAVA_OP_NONE },  // aconst_null (0x01)
  { "iconst_m1",       1,  0,  JAVA_OP_NONE },  // iconst_m1 (0x02)
  { "iconst_0",        1,  0,  JAVA_OP_NONE },  // iconst_0 (0x03)
  { "iconst_1",        1,  0,  JAVA_OP_NONE },  // iconst_1 (0x04)
  { "iconst_2",        1,  0,  JAVA_OP_NONE },  // iconst_2 (0x05)
  { "iconst_3",        1,  0,  JAVA_OP_NONE },  // iconst_3 (0x06)
  { "iconst_4",        1,  0,  JAVA_OP_NONE },  // iconst_4 (0x07)
  { "iconst_5",        1,  0,  JAVA_OP_NONE },  // iconst_5 (0x08)
  { "lconst_0",        1,  0,  JAVA_OP_NONE },  // lconst_0 (0x09)
  { "lconst_1",        1,  0,  JAVA_OP_NONE },  // lconst_1 (0x0a)
  { "fconst_0",        1,  0,  JAVA_OP_NONE },  // fconst_0 (0x0b)
  { "fconst_1",        1,  0,  JAVA_OP_NONE },  // fconst_1 (0x0c)
  { "fconst_2",        1,  0,  JAVA_OP_NONE },  // fconst_2 (0x0d)
  { "dconst_0",        1,  0,  JAVA_OP_NONE },  // dconst_0 (0x0e)
  { "dconst_1",        1,  0,  JAVA_OP_NONE },  // dconst_1 (0x0f)
  { "bipush",          2,  0,  JAVA_OP_CONSTANT8 },  // bipush (0x10)
  { "sipush",          3,  0,  JAVA_OP_CONSTANT16 },  // sipush (0x11)
  { "ldc",             2,  0,  JAVA_OP_CONSTANT_INDEX8 },  // ldc (0x12)
  { "ldc_w",           3,  0,  JAVA_OP_CONSTANT_INDEX },  // ldc_w (0x13)
  { "ldc2_w",          3,  0,  JAVA_OP_CONSTANT_INDEX },  // ldc2_w (0x14)
  { "iload",           2,  3,  JAVA_OP_LOCAL_INDEX },  // iload (0x15)
  { "lload",           2,  3,  JAVA_OP_LOCAL_INDEX },  // lload (0x16)
  { "fload",           2,  3,  JAVA_OP_LOCAL_INDEX },  // fload (0x17)
  { "dload",           2,  3,  JAVA_OP_LOCAL_INDEX },  // dload (0x18)
  { "aload",           2,  3,  JAVA_OP_LOCAL_INDEX },  // aload (0x19)
  { "iload_0",         1,  0,  JAVA_OP_NONE },  // iload_0 (0x1a)
  { "iload_1",         1,  0,  JAVA_OP_NONE },  // iload_1 (0x1b)
  { "iload_2",         1,  0,  JAVA_OP_NONE },  // iload_2 (0x1c)
  { "iload_3",         1,  0,  JAVA_OP_NONE },  // iload_3 (0x1d)
  { "lload_0",         1,  0,  JAVA_OP_NONE },  // lload_0 (0x1e)
  { "lload_1",         1,  0,  JAVA_OP_NONE },  // lload_1 (0x1f)
  { "lload_2",         1,  0,  JAVA_OP_NONE },  // lload_2 (0x20)
  { "lload_3",         1,  0,  JAVA_OP_NONE },  // lload_3 (0x21)
  { "fload_0",         1,  0,  JAVA_OP_NONE },  // fload_0 (0x22)
  { "fload_1",         1,  0,  JAVA_OP_NONE },  // fload_1 (0x23)
  { "fload_2",         1,  0,  JAVA_OP_NONE },  // fload_2 (0x24)
  { "fload_3",         1,  0,  JAVA_OP_NONE },  // fload_3 (0x25)
  { "dload_0",         1,  0,  JAVA_OP_NONE },  // dload_0 (0x26)
  { "dload_1",         1,  0,  JAVA_OP_NONE },  // dload_1 (0x27)
  { "dload_2",         1,  0,  JAVA_OP_NONE },  // dload_2 (0x28)
  { "dload_3",         1,  0,  JAVA_OP_NONE },  // dload_3 (0x29)
  { "aload_0",         1,  0,  JAVA_OP_NONE },  // aload_0 (0x2a)
  { "aload_1",         1,  0,  JAVA_OP_NONE },  // aload_1 (0x2b)
  { "aload_2",         1,  0,  JAVA_OP_NONE },  // aload_2 (0x2c)
  { "aload_3",         1,  0,  JAVA_OP_NONE },  // aload_3 (0x2d)
  { "iaload",          1,  0,  JAVA_OP_NONE },  // iaload (0x2e)
  { "laload",          1,  0,  JAVA_OP_NONE },  // laload (0x2f)
  { "faload",          1,  0,  JAVA_OP_NONE },  // faload (0x30)
  { "daload",          1,  0,  JAVA_OP_NONE },  // daload (0x31)
  { "aaload",          1,  0,  JAVA_OP_NONE },  // aaload (0x32)
  { "baload",          1,  0,  JAVA_OP_NONE },  // baload (0x33)
  { "caload",          1,  0,  JAVA_OP_NONE },  // caload (0x34)
  { "saload",          1,  0,  JAVA_OP_NONE },  // saload (0x35)
  { "istore",          2,  3,  JAVA_OP_LOCAL_INDEX },  // istore (0x36)
  { "lstore",          2,  3,  JAVA_OP_LOCAL_INDEX },  // lstore (0x37)
  { "fstore",          2,  3,  JAVA_OP_LOCAL_INDEX },  // fstore (0x38)
  { "dstore",          2,  3,  JAVA_OP_LOCAL_INDEX },  // dstore (0x39)
  { "astore",          2,  3,  JAVA_OP_LOCAL_INDEX },  // astore (0x3a)
  { "istore_0",        1,  0,  JAVA_OP_NONE },  // istore_0 (0x3b)
  { "istore_1",        1,  0,  JAVA_OP_NONE },  // istore_1 (0x3c)
  { "istore_2",        1,  0,  JAVA_OP_NONE },  // istore_2 (0x3d)
  { "istore_3",        1,  0,  JAVA_OP_NONE },  // istore_3 (0x3e)
  { "lstore_0",        1,  0,  JAVA_OP_NONE },  // lstore_0 (0x3f)
  { "lstore_1",        1,  0,  JAVA_OP_NONE },  // lstore_1 (0x40)
  { "lstore_2",        1,  0,  JAVA_OP_NONE },  // lstore_2 (0x41)
  { "lstore_3",        1,  0,  JAVA_OP_NONE },  // lstore_3 (0x42)
  { "fstore_0",        1,  0,  JAVA_OP_NONE },  // fstore_0 (0x43)
  { "fstore_1",        1,  0,  JAVA_OP_NONE },  // fstore_1 (0x44)
  { "fstore_2",        1,  0,  JAVA_OP_NONE },  // fstore_2 (0x45)
  { "fstore_3",        1,  0,  JAVA_OP_NONE },  // fstore_3 (0x46)
  { "dstore_0",        1,  0,  JAVA_OP_NONE },  // dstore_0 (0x47)
  { "dstore_1",        1,  0,  JAVA_OP_NONE },  // dstore_1 (0x48)
  { "dstore_2",        1,  0,  JAVA_OP_NONE },  // dstore_2 (0x49)
  { "dstore_3",        1,  0,  JAVA_OP_NONE },  // dstore_3 (0x4a)
  { "astore_0",        1,  0,  JAVA_OP_NONE },  // astore_0 (0x4b)
  { "astore_1",        1,  0,  JAVA_OP_NONE },  // astore_1 (0x4c)
  { "astore_2",        1,  0,  JAVA_OP_NONE },  // astore_2 (0x4d)
  { "astore_3",        1,  0,  JAVA_OP_NONE },  // astore_3 (0x4e)
  { "iastore",         1,  0,  JAVA_OP_NONE },  // iastore (0x4f)
  { "lastore",         1,  0,  JAVA_OP_NONE },  // lastore (0x50)
  { "fastore",         1,  0,  JAVA_OP_NONE },  // fastore (0x51)
  { "dastore",         1,  0,  JAVA_OP_NONE },  // dastore (0x52)
  { "aastore",         1,  0,  JAVA_OP_NONE },  // aastore (0x53)
  { "bastore",         1,  0,  JAVA_OP_NONE },  // bastore (0x54)
  { "castore",         1,  0,  JAVA_OP_NONE },  // castore (0x55)
  { "sastore",         1,  0,  JAVA_OP_NONE },  // sastore (0x56)
  { "pop",             1,  0,  JAVA_OP_NONE },  // pop (0x57)
  { "pop2",            1,  0,  JAVA_OP_NONE },  // pop2 (0x58)
  { "dup",             1,  0,  JAVA_OP_NONE },  // dup (0x59)
  { "dup_x1",          1,  0,  JAVA_OP_NONE },  // dup_x1 (0x5a)
  { "dup_x2",          1,  0,  JAVA_OP_NONE },  // dup_x2 (0x5b)
  { "dup2",            1,  0,  JAVA_OP_NONE },  // dup2 (0x5c)
  { "dup2_x1",         1,  0,  JAVA_OP_NONE },  // dup2_x1 (0x5d)
  { "dup2_x2",         1,  0,  JAVA_OP_NONE },  // dup2_x2 (0x5e)
  { "swap",            1,  0,  JAVA_OP_NONE },  // swap (0x5f)
  { "iadd",            1,  0,  JAVA_OP_NONE },  // iadd (0x60)
  { "ladd",            1,  0,  JAVA_OP_NONE },  // ladd (0x61)
  { "fadd",            1,  0,  JAVA_OP_NONE },  // fadd (0x62)
  { "dadd",            1,  0,  JAVA_OP_NONE },  // dadd (0x63)
  { "isub",            1,  0,  JAVA_OP_NONE },  // isub (0x64)
  { "lsub",            1,  0,  JAVA_OP_NONE },  // lsub (0x65)
  { "fsub",            1,  0,  JAVA_OP_NONE },  // fsub (0x66)
  { "dsub",            1,  0,  JAVA_OP_NONE },  // dsub (0x67)
  { "imul",            1,  0,  JAVA_OP_NONE },  // imul (0x68)
  { "lmul",            1,  0,  JAVA_OP_NONE },  // lmul (0x69)
  { "fmul",            1,  0,  JAVA_OP_NONE },  // fmul (0x6a)
  { "dmul",            1,  0,  JAVA_OP_NONE },  // dmul (0x6b)
  { "idiv",            1,  0,  JAVA_OP_NONE },  // idiv (0x6c)
  { "ldiv",            1,  0,  JAVA_OP_NONE },  // ldiv (0x6d)
  { "fdiv",            1,  0,  JAVA_OP_NONE },  // fdiv (0x6e)
  { "ddiv",            1,  0,  JAVA_OP_NONE },  // ddiv (0x6f)
  { "irem",            1,  0,  JAVA_OP_NONE },  // irem (0x70)
  { "lrem",            1,  0,  JAVA_OP_NONE },  // lrem (0x71)
  { "frem",            1,  0,  JAVA_OP_NONE },  // frem (0x72)
  { "drem",            1,  0,  JAVA_OP_NONE },  // drem (0x73)
  { "ineg",            1,  0,  JAVA_OP_NONE },  // ineg (0x74)
  { "lneg",            1,  0,  JAVA_OP_NONE },  // lneg (0x75)
  { "fneg",            1,  0,  JAVA_OP_NONE },  // fneg (0x76)
  { "dneg",            1,  0,  JAVA_OP_NONE },  // dneg (0x77)
  { "ishl",            1,  0,  JAVA_OP_NONE },  // ishl (0x78)
  { "lshl",            1,  0,  JAVA_OP_NONE },  // lshl (0x79)
  { "ishr",            1,  0,  JAVA_OP_NONE },  // ishr (0x7a)
  { "lshr",            1,  0,  JAVA_OP_NONE },  // lshr (0x7b)
  { "iushr",           1,  0,  JAVA_OP_NONE },  // iushr (0x7c)
  { "lushr",           1,  0,  JAVA_OP_NONE },  // lushr (0x7d)
  { "iand",            1,  0,  JAVA_OP_NONE },  // iand (0x7e)
  { "land",            1,  0,  JAVA_OP_NONE },  // land (0x7f)
  { "ior",             1,  0,  JAVA_OP_NONE },  // ior (0x80)
  { "lor",             1,  0,  JAVA_OP_NONE },  // lor (0x81)
  { "ixor",            1,  0,  JAVA_OP_NONE },  // ixor (0x82)
  { "lxor",            1,  0,  JAVA_OP_NONE },  // lxor (0x83)
  { "iinc",            3,  5,  JAVA_OP_LOCAL_INDEX_CONST },  // iinc (0x84)
  { "i2l",             1,  0,  JAVA_OP_NONE },  // i2l (0x85)
  { "i2f",             1,  0,  JAVA_OP_NONE },  // i2f (0x86)
  { "i2d",             1,  0,  JAVA_OP_NONE },  // i2d (0x87)
  { "l2i",             1,  0,  JAVA_OP_NONE },  // l2i (0x88)
  { "l2f",             1,  0,  JAVA_OP_NONE },  // l2f (0x89)
  { "l2d",             1,  0,  JAVA_OP_NONE },  // l2d (0x8a)
  { "f2i",             1,  0,  JAVA_OP_NONE },  // f2i (0x8b)
  { "f2l",             1,  0,  JAVA_OP_NONE },  // f2l (0x8c)
  { "f2d",             1,  0,  JAVA_OP_NONE },  // f2d (0x8d)
  { "d2i",             1,  0,  JAVA_OP_NONE },  // d2i (0x8e)
  { "d2l",             1,  0,  JAVA_OP_NONE },  // d2l (0x8f)
  { "d2f",             1,  0,  JAVA_OP_NONE },  // d2f (0x90)
  { "i2b",             1,  0,  JAVA_OP_NONE },  // i2b (0x91)
  { "i2c",             1,  0,  JAVA_OP_NONE },  // i2c (0x92)
  { "i2s",             1,  0,  JAVA_OP_NONE },  // i2s (0x93)
  { "lcmp",            1,  0,  JAVA_OP_NONE },  // lcmp (0x94)
  { "fcmpl",           1,  0,  JAVA_OP_NONE },  // fcmpl (0x95)
  { "fcmpg",           1,  0,  JAVA_OP_NONE },  // fcmpg (0x96)
  { "dcmpl",           1,  0,  JAVA_OP_NONE },  // dcmpl (0x97)
  { "dcmpg",           1,  0,  JAVA_OP_NONE },  // dcmpg (0x98)
  { "ifeq",            3,  0,  JAVA_OP_OFFSET16 },  // ifeq (0x99)
  { "ifne",            3,  0,  JAVA_OP_OFFSET16 },  // ifne (0x9a)
  { "iflt",            3,  0,  JAVA_OP_OFFSET16 },  // iflt (0x9b)
  { "ifge",            3,  0,  JAVA_OP_OFFSET16 },  // ifge (0x9c)
  { "ifgt",            3,  0,  JAVA_OP_OFFSET16 },  // ifgt (0x9d)
  { "ifle",            3,  0,  JAVA_OP_OFFSET16 },  // ifle (0x9e)
  { "if_icmpeq",       3,  0,  JAVA_OP_OFFSET16 },  // if_icmpeq (0x9f)
  { "if_icmpne",       3,  0,  JAVA_OP_OFFSET16 },  // if_icmpne (0xa0)
  { "if_icmplt",       3,  0,  JAVA_OP_OFFSET16 },  // if_icmplt (0xa1)
  { "if_icmpge",       3,  0,  JAVA_OP_OFFSET16 },  // if_icmpge (0xa2)
  { "if_icmpgt",       3,  0,  JAVA_OP_OFFSET16 },  // if_icmpgt (0xa3)
  { "if_icmple",       3,  0,  JAVA_OP_OFFSET16 },  // if_icmple (0xa4)
  { "if_acmpeq",       1,  0,  JAVA_OP_OFFSET16 },  // if_acmpeq (0xa5)
  { "if_acmpne",       1,  0,  JAVA_OP_OFFSET16 },  // if_acmpne (0xa6)
  { "goto",            3,  0,  JAVA_OP_OFFSET16 },  // goto (0xa7)
  { "jsr",             3,  0,  JAVA_OP_OFFSET16 },  // jsr (0xa8)
  { "ret",             2,  3,  JAVA_OP_LOCAL_INDEX },  // ret (0xa9)
  { "tableswitch",     1,  0,  JAVA_OP_NONE },  // tableswitch (0xaa)
  { "lookupswitch",    1,  0,  JAVA_OP_NONE },  // lookupswitch (0xab)
  { "ireturn",         1,  0,  JAVA_OP_NONE },  // ireturn (0xac)
  { "lreturn",         1,  0,  JAVA_OP_NONE },  // lreturn (0xad)
  { "freturn",         1,  0,  JAVA_OP_NONE },  // freturn (0xae)
  { "dreturn",         1,  0,  JAVA_OP_NONE },  // dreturn (0xaf)
  { "areturn",         1,  0,  JAVA_OP_NONE },  // areturn (0xb0)
  { "return",          1,  0,  JAVA_OP_NONE },  // return (0xb1)
  { "getstatic",       3,  0,  JAVA_OP_STATIC_INDEX },  // getstatic (0xb2)
  { "putstatic",       3,  0,  JAVA_OP_STATIC_INDEX },  // putstatic (0xb3)
  { "getfield",        3,  0,  JAVA_OP_FIELD_INDEX },  // getfield (0xb4)
  { "putfield",        3,  0,  JAVA_OP_FIELD_INDEX },  // putfield (0xb5)
  { "invokevirtual",   3,  0,  JAVA_OP_VIRTUAL_INDEX },  // invokevirtual (0xb6)
  { "invokespecial",   3,  0,  JAVA_OP_SPECIAL_INDEX },  // invokespecial (0xb7)
  { "invokestatic",    3,  0,  JAVA_OP_STATIC_INDEX },  // invokestatic (0xb8)
  { "invokeinterface", 3,  0,  JAVA_OP_INTERFACE_INDEX }, // invokeinterface (0xb9)
  { "invokedynamic",   3,  0,  JAVA_OP_METHOD_INDEX },  // invokedynamic (0xba)
  { "new",             3,  0,  JAVA_OP_CLASS_INDEX },  // new (0xbb)
  { "newarray",        2,  0,  JAVA_OP_ARRAY_TYPE },  // newarray (0xbc)
  { "anewarray",       3,  0,  JAVA_OP_CLASS_INDEX },  // anewarray (0xbd)
  { "arraylength",     1,  0,  JAVA_OP_NONE },  // arraylength (0xbe)
  { "athrow",          1,  0,  JAVA_OP_NONE },  // athrow (0xbf)
  { "checkcast",       1,  0,  JAVA_OP_NONE },  // checkcast (0xc0)
  { "instanceof",      1,  0,  JAVA_OP_NONE },  // instanceof (0xc1)
  { "monitorenter",    1,  0,  JAVA_OP_NONE },  // monitorenter (0xc2)
  { "monitorexit",     1,  0,  JAVA_OP_NONE },  // monitorexit (0xc3)
  { "wide",            1,  0,  JAVA_OP_ILLEGAL },  // wide (0xc4)
  { "multianewarray",  1,  0,  JAVA_OP_NONE },  // multianewarray (0xc5)
  { "ifnull",          3,  0,  JAVA_OP_OFFSET16 },  // ifnull (0xc6)
  { "ifnonnull",       3,  0,  JAVA_OP_OFFSET16 },  // ifnonnull (0xc7)
  { "goto_w",          5,  0,  JAVA_OP_OFFSET32 },  // goto_w (0xc8)
  { "jsr_w",           5,  0,  JAVA_OP_OFFSET32 },  // jsr_w (0xc9)
  { "breakpoint",      1,  0,  JAVA_OP_NONE },  // breakpoint (0xca)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xcb)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xcc)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xcd)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xce)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xcf)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd0)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd1)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd2)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd3)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd4)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd5)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd6)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd7)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd8)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xd9)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xda)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xdb)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xdc)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xdd)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xde)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xdf)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe0)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe1)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe2)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe3)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe4)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe5)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe6)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe7)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe8)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xe9)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xea)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xeb)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xec)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xed)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xee)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xef)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf0)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf1)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf2)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf3)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf4)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf5)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf6)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf7)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf8)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xf9)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xfa)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xfb)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xfc)
  { "???",             0,  0,  JAVA_OP_ILLEGAL },  // undefined (0xfd)
  { "impdep1",         1,  0,  JAVA_OP_WARN },  // impdep1 (0xfe)
  { "impdep2",         1,  0,  JAVA_OP_WARN },  // impdep2 (0xff)
};

