/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2019 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "disasm/stm8.h"
#include "table/stm8.h"

struct _table_stm8 table_stm8[] = {
  { "adc", STM8_ADC },
  { "add", STM8_ADD },
  { "addw", STM8_ADDW },
  { "and", STM8_AND },
  { "bccm", STM8_BCCM },
  { "bcp", STM8_BCP },
  { "bcpl", STM8_BCPL },
  { "break", STM8_BREAK },
  { "bres", STM8_BRES },
  { "bset", STM8_BSET },
  { "btjf", STM8_BTJF },
  { "btjt", STM8_BTJT },
  { "call", STM8_CALL },
  { "callf", STM8_CALLF },
  { "callr", STM8_CALLR },
  { "ccf", STM8_CCF },
  { "clr", STM8_CLR },
  { "clrw", STM8_CLRW },
  { "cp", STM8_CP },
  { "cpw", STM8_CPW },
  { "cpl", STM8_CPL },
  { "cplw", STM8_CPLW },
  { "dec", STM8_DEC },
  { "decw", STM8_DECW },
  { "div", STM8_DIV },
  { "divw", STM8_DIVW },
  { "exg", STM8_EXG },
  { "exgw", STM8_EXGW },
  { "halt", STM8_HALT },
  { "inc", STM8_INC },
  { "incw", STM8_INCW },
  { "int", STM8_INT },
  { "iret", STM8_IRET },
  { "jp", STM8_JP },
  { "jpf", STM8_JPF },
  { "jra", STM8_JRA },
  { "jrc", STM8_JRC },
  { "jreq", STM8_JREQ },
  { "jrf", STM8_JRF },
  { "jrh", STM8_JRH },
  { "jrih", STM8_JRIH },
  { "jril", STM8_JRIL },
  { "jrm", STM8_JRM },
  { "jrmi", STM8_JRMI },
  { "jrnc", STM8_JRNC },
  { "jrne", STM8_JRNE },
  { "jrnh", STM8_JRNH },
  { "jrnm", STM8_JRNM },
  { "jrnv", STM8_JRNV },
  { "jrpl", STM8_JRPL },
  { "jrsge", STM8_JRSGE },
  { "jrsgt", STM8_JRSGT },
  { "jrsle", STM8_JRSLE },
  { "jrslt", STM8_JRSLT },
  { "jrt", STM8_JRT },
  { "jruge", STM8_JRUGE },
  { "jrugt", STM8_JRUGT },
  { "jrule", STM8_JRULE },
  //{ "jrc", STM8_JRC },
  { "jrult", STM8_JRULT },
  { "jrv", STM8_JRV },
  { "ld", STM8_LD },
  { "ldf", STM8_LDF },
  { "ldw", STM8_LDW },
  { "mov", STM8_MOV },
  { "mul", STM8_MUL },
  { "neg", STM8_NEG },
  { "negw", STM8_NEGW },
  { "nop", STM8_NOP },
  { "or", STM8_OR },
  { "pop", STM8_POP },
  { "popw", STM8_POPW },
  { "push", STM8_PUSH },
  { "pushw", STM8_PUSHW },
  { "rcf", STM8_RCF },
  { "ret", STM8_RET },
  { "retf", STM8_RETF },
  { "rim", STM8_RIM },
  { "rlc", STM8_RLC },
  { "rlcw", STM8_RLCW },
  { "rlwa", STM8_RLWA },
  { "rrc", STM8_RRC },
  { "rrcw", STM8_RRCW },
  { "rrwa", STM8_RRWA },
  { "rvf", STM8_RVF },
  { "sbc", STM8_SBC },
  { "scf", STM8_SCF },
  { "sim", STM8_SIM },
  { "sll", STM8_SLL },
  { "sla", STM8_SLA },
  { "sllw", STM8_SLLW },
  { "slaw", STM8_SLAW },
  { "sra", STM8_SRA },
  { "sraw", STM8_SRAW },
  { "srl", STM8_SRL },
  { "srlw", STM8_SRLW },
  { "sub", STM8_SUB },
  { "subw", STM8_SUBW },
  { "swap", STM8_SWAP },
  { "swapw", STM8_SWAPW },
  { "tnz", STM8_TNZ },
  { "tnzw", STM8_TNZW },
  { "trap", STM8_TRAP },
  { "wfe", STM8_WFE },
  { "wfi", STM8_WFI },
  { "xor", STM8_XOR },
  { NULL, 0 }
};

// October 21, 2014
// teqo: please call the table bigHonkinTable
struct _table_stm8_opcodes table_stm8_opcodes[] = {
  // ADC
  { STM8_ADC, OP_NUMBER8, 0, 0xa9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_ADDRESS8, 0, 0xb9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_ADDRESS16, 0, 0xc9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_INDEX_X, 0, 0xf9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_OFFSET8_INDEX_X, 0, 0xe9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_OFFSET16_INDEX_X, 0, 0xd9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_INDEX_Y, 0x90, 0xf9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_OFFSET8_INDEX_Y, 0x90, 0xe9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_OFFSET16_INDEX_Y, 0x90, 0xd9, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADC, OP_OFFSET8_INDEX_SP, 0, 0x19, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_ADC, OP_INDIRECT8, 0x92, 0xc9, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_ADC, OP_INDIRECT16, 0x72, 0xc9, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_ADC, OP_INDIRECT8_X, 0x92, 0xd9, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_ADC, OP_INDIRECT16_X, 0x72, 0xd9, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_ADC, OP_INDIRECT8_Y, 0x91, 0xd9, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  // ADD
  { STM8_ADD, OP_NUMBER8, 0, 0xab, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_ADDRESS8, 0, 0xbb, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_ADDRESS16, 0, 0xcb, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_INDEX_X, 0, 0xfb, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_OFFSET8_INDEX_X, 0, 0xeb, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_OFFSET16_INDEX_X, 0, 0xdb, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_INDEX_Y, 0x90, 0xfb, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_OFFSET8_INDEX_Y, 0x90, 0xeb, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_OFFSET16_INDEX_Y, 0x90, 0xdb, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_ADD, OP_OFFSET8_INDEX_SP, 0, 0x1b, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_ADD, OP_INDIRECT8, 0x92, 0xcb, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_ADD, OP_INDIRECT16, 0x72, 0xcb, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_ADD, OP_INDIRECT8_X, 0x92, 0xdb, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_ADD, OP_INDIRECT16_X, 0x72, 0xdb, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_ADD, OP_INDIRECT8_Y, 0x91, 0xdb, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  // ADDW
  { STM8_ADDW, OP_NUMBER16, 0, 0x1c, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_ADDW, OP_ADDRESS16, 0x72, 0xbb, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_ADDW, OP_OFFSET8_INDEX_SP, 0x72, 0xfb, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_ADDW, OP_NUMBER16, 0x72, 0xa9, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  { STM8_ADDW, OP_ADDRESS16, 0x72, 0xb9, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  { STM8_ADDW, OP_OFFSET8_INDEX_SP, 0x72, 0xf9, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  { STM8_ADDW, OP_NUMBER8, 0, 0x5b, OP_SP, OP_NONE, 2,2, ST7_NO },
  // AND
  { STM8_AND, OP_NUMBER8, 0, 0xa4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_ADDRESS8, 0, 0xb4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_ADDRESS16, 0, 0xc4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_INDEX_X, 0, 0xf4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_OFFSET8_INDEX_X, 0, 0xe4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_OFFSET16_INDEX_X, 0, 0xd4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_INDEX_Y, 0x90, 0xf4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_OFFSET8_INDEX_Y, 0x90, 0xe4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_OFFSET16_INDEX_Y, 0x90, 0xd4, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_AND, OP_OFFSET8_INDEX_SP, 0, 0x14, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_AND, OP_INDIRECT8, 0x92, 0xc4, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_AND, OP_INDIRECT16, 0x72, 0xc4, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_AND, OP_INDIRECT8_X, 0x92, 0xd4, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_AND, OP_INDIRECT16_X, 0x72, 0xd4, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_AND, OP_INDIRECT8_Y, 0x91, 0xd4, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  // BCCM
  { STM8_BCCM, OP_ADDRESS_BIT, 0x90, 0x11, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // BCP
  { STM8_BCP, OP_NUMBER8, 0, 0xa5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_ADDRESS8, 0, 0xb5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_ADDRESS16, 0, 0xc5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_INDEX_X, 0, 0xf5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_OFFSET8_INDEX_X, 0, 0xe5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_OFFSET16_INDEX_X, 0, 0xd5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_INDEX_Y, 0x90, 0xf5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_OFFSET8_INDEX_Y, 0x90, 0xe5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_OFFSET16_INDEX_Y, 0x90, 0xd5, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_BCP, OP_OFFSET8_INDEX_SP, 0, 0x15, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_BCP, OP_INDIRECT8, 0x92, 0xc5, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_BCP, OP_INDIRECT16, 0x72, 0xc5, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_BCP, OP_INDIRECT8_X, 0x92, 0xd5, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_BCP, OP_INDIRECT16_X, 0x72, 0xd5, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_BCP, OP_INDIRECT8_Y, 0x91, 0xd5, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  // BCPL
  { STM8_BCPL, OP_ADDRESS_BIT, 0x90, 0x10, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // BREAK
  { STM8_BREAK, OP_NONE, 0, 0x8B, OP_NONE, OP_NONE, 1,1, ST7_YES },
  // BRES
  { STM8_BRES, OP_ADDRESS_BIT, 0x72, 0x11, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // BSET
  { STM8_BSET, OP_ADDRESS_BIT, 0x72, 0x10, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // BTJF
  { STM8_BTJF, OP_ADDRESS_BIT_LOOP, 0x72, 0x01, OP_NONE, OP_NONE, 2,3, ST7_NO },
  // BTJT
  { STM8_BTJT, OP_ADDRESS_BIT_LOOP, 0x72, 0x00, OP_NONE, OP_NONE, 2,3, ST7_NO },
  // CALL
  { STM8_CALL, OP_ADDRESS16, 0, 0xcd, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CALL, OP_INDEX_X, 0, 0xfd, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CALL, OP_OFFSET8_INDEX_X, 0, 0xed, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CALL, OP_OFFSET16_INDEX_X, 0, 0xdd, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CALL, OP_INDEX_Y, 0x90, 0xfd, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CALL, OP_OFFSET8_INDEX_Y, 0x90, 0xed, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CALL, OP_OFFSET16_INDEX_Y, 0x90, 0xdd, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CALL, OP_INDIRECT8, 0x92, 0xcd, OP_NONE, OP_NONE, 6,6, ST7_YES },
  { STM8_CALL, OP_INDIRECT16, 0x72, 0xcd, OP_NONE, OP_NONE, 6,6, ST7_NO },
  { STM8_CALL, OP_INDIRECT8_X, 0x92, 0xdd, OP_NONE, OP_NONE, 6,6, ST7_YES },
  { STM8_CALL, OP_INDIRECT16_X, 0x72, 0xdd, OP_NONE, OP_NONE, 6,6, ST7_NO },
  { STM8_CALL, OP_INDIRECT8_Y, 0x91, 0xdd, OP_NONE, OP_NONE, 6,6, ST7_YES },
  // CALLF
  { STM8_CALLF, OP_ADDRESS24, 0, 0x8d, OP_NONE, OP_NONE, 5,5, ST7_NO },
  { STM8_CALLF, OP_INDIRECT16_E, 0x92, 0x8d, OP_NONE, OP_NONE, 8,8, ST7_NO },
  // CALLR
  { STM8_CALLR, OP_RELATIVE, 0, 0xad, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // CCF
  { STM8_CCF, OP_NONE, 0, 0x8C, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // CLR
  { STM8_CLR, OP_SINGLE_REGISTER, 0, 0x4f, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CLR, OP_ADDRESS8, 0, 0x3f, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CLR, OP_ADDRESS16, 0x72, 0x5f, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_CLR, OP_INDEX_X, 0, 0x7f, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CLR, OP_OFFSET8_INDEX_X, 0, 0x6f, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CLR, OP_OFFSET16_INDEX_X, 0x72, 0x4f, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_CLR, OP_INDEX_Y, 0x90, 0x7f, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CLR, OP_OFFSET8_INDEX_Y, 0x90, 0x6f, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CLR, OP_OFFSET16_INDEX_Y, 0x90, 0x4f, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_CLR, OP_OFFSET8_INDEX_SP, 0, 0x0f, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_CLR, OP_INDIRECT8, 0x92, 0x3f, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CLR, OP_INDIRECT16, 0x72, 0x3f, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_CLR, OP_INDIRECT8_X, 0x92, 0x6f, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CLR, OP_INDIRECT16_X, 0x72, 0x6f, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_CLR, OP_INDIRECT8_Y, 0x91, 0x6f, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // CLRW
  { STM8_CLRW, OP_SINGLE_REGISTER, 0, 0x5f, OP_REG_X, OP_NONE, 1,1, ST7_NO },
  { STM8_CLRW, OP_SINGLE_REGISTER, 0x90, 0x5f, OP_REG_Y, OP_NONE, 1,1, ST7_NO },
  // CP
  { STM8_CP, OP_NUMBER8, 0, 0xa1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_ADDRESS8, 0, 0xb1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_ADDRESS16, 0, 0xc1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_INDEX_X, 0, 0xf1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_OFFSET8_INDEX_X, 0, 0xe1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_OFFSET16_INDEX_X, 0, 0xd1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_INDEX_Y, 0x90, 0xf1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_OFFSET8_INDEX_Y, 0x90, 0xe1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_OFFSET16_INDEX_Y, 0x90, 0xd1, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CP, OP_OFFSET8_INDEX_SP, 0, 0x11, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_CP, OP_INDIRECT8, 0x92, 0xc1, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_CP, OP_INDIRECT16, 0x72, 0xc1, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_CP, OP_INDIRECT8_X, 0x92, 0xd1, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_CP, OP_INDIRECT16_X, 0x72, 0xd1, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_CP, OP_INDIRECT8_Y, 0x91, 0xd1, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  // CPW
  { STM8_CPW, OP_NUMBER16, 0, 0xa3, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_ADDRESS8, 0, 0xb3, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_ADDRESS16, 0, 0xc3, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_INDEX_Y, 0x90, 0xf3, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_OFFSET8_INDEX_Y, 0x90, 0xe3, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_OFFSET16_INDEX_Y, 0x90, 0xd3, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_OFFSET8_INDEX_SP, 0, 0x13, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_CPW, OP_INDIRECT8, 0x92, 0xc3, OP_REG_X, OP_NONE, 5,5, ST7_YES },
  { STM8_CPW, OP_INDIRECT16, 0x72, 0xc3, OP_REG_X, OP_NONE, 5,5, ST7_NO },
  { STM8_CPW, OP_INDIRECT8_Y, 0x91, 0xd3, OP_REG_X, OP_NONE, 5,5, ST7_YES },

  { STM8_CPW, OP_NUMBER16, 0x90, 0xa3, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_ADDRESS8, 0x90, 0xb3, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_ADDRESS16, 0x90, 0xc3, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_INDEX_X, 0, 0xf3, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_OFFSET8_INDEX_X, 0, 0xe3, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_OFFSET16_INDEX_X, 0, 0xd3, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_CPW, OP_INDIRECT8, 0x91, 0xc3, OP_REG_Y, OP_NONE, 5,5, ST7_YES },
  { STM8_CPW, OP_INDIRECT8_X, 0x92, 0xd3, OP_REG_Y, OP_NONE, 5,5, ST7_NO },
  { STM8_CPW, OP_INDIRECT16_X, 0x72, 0xd3, OP_REG_Y, OP_NONE, 5,5, ST7_YES },
  // CPL
  { STM8_CPL, OP_SINGLE_REGISTER, 0, 0x43, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_CPL, OP_ADDRESS8, 0, 0x33, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CPL, OP_ADDRESS16, 0x72, 0x53, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_CPL, OP_INDEX_X, 0, 0x73, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CPL, OP_OFFSET8_INDEX_X, 0, 0x63, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CPL, OP_OFFSET16_INDEX_X, 0x72, 0x43, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_CPL, OP_INDEX_Y, 0x90, 0x73, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CPL, OP_OFFSET8_INDEX_Y, 0x90, 0x63, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CPL, OP_OFFSET16_INDEX_Y, 0x90, 0x43, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_CPL, OP_OFFSET8_INDEX_SP, 0, 0x03, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_CPL, OP_INDIRECT8, 0x92, 0x33, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CPL, OP_INDIRECT16, 0x72, 0x33, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_CPL, OP_INDIRECT8_X, 0x92, 0x63, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_CPL, OP_INDIRECT16_X, 0x72, 0x63, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_CPL, OP_INDIRECT8_Y, 0x91, 0x63, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // CPLW
  { STM8_CPLW, OP_SINGLE_REGISTER, 0, 0x53, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_CPLW, OP_SINGLE_REGISTER, 0x90, 0x53, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  // DEC
  // FIXME: .w is outside the [] or non-existent.  Is this a typo in the
  // datasheet?
  { STM8_DEC, OP_SINGLE_REGISTER, 0, 0x4a, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_DEC, OP_ADDRESS8, 0, 0x3a, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_DEC, OP_ADDRESS16, 0x72, 0x5a, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_DEC, OP_INDEX_X, 0, 0x7a, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_DEC, OP_OFFSET8_INDEX_X, 0, 0x6a, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_DEC, OP_OFFSET16_INDEX_X, 0x72, 0x4a, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_DEC, OP_INDEX_Y, 0x90, 0x7a, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_DEC, OP_OFFSET8_INDEX_Y, 0x90, 0x6a, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_DEC, OP_OFFSET16_INDEX_Y, 0x90, 0x4a, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_DEC, OP_OFFSET8_INDEX_SP, 0, 0x0a, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_DEC, OP_INDIRECT8, 0x92, 0x3a, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_DEC, OP_INDIRECT16, 0x72, 0x3a, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_DEC, OP_INDIRECT8_X, 0x92, 0x6a, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_DEC, OP_INDIRECT16_X, 0x72, 0x6a, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_DEC, OP_INDIRECT8_Y, 0x91, 0x6a, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // DECW
  { STM8_DECW, OP_SINGLE_REGISTER, 0, 0x5a, OP_REG_X, OP_NONE, 1,1, ST7_NO },
  { STM8_DECW, OP_SINGLE_REGISTER, 0x90, 0x5a, OP_REG_Y, OP_NONE, 1,1, ST7_NO },
  // DIV
  { STM8_DIV, OP_TWO_REGISTERS, 0, 0x62, OP_REG_X, OP_REG_A, 2,17, ST7_NO },
  { STM8_DIV, OP_TWO_REGISTERS, 0x90, 0x62, OP_REG_Y, OP_REG_A, 2,17, ST7_NO },
  // DIVW
  { STM8_DIVW, OP_TWO_REGISTERS, 0, 0x65, OP_REG_X, OP_REG_Y, 2,17, ST7_NO },
  // EXG
  { STM8_EXG, OP_TWO_REGISTERS, 0, 0x41, OP_REG_A, OP_REG_XL, 1,1, ST7_NO },
  { STM8_EXG, OP_TWO_REGISTERS, 0, 0x61, OP_REG_A, OP_REG_YL, 1,1, ST7_NO },
  { STM8_EXG, OP_ADDRESS16, 0, 0x31, OP_REG_A, OP_NONE, 3,3, ST7_NO },
  // EXGW
  { STM8_EXGW, OP_TWO_REGISTERS, 0, 0x51, OP_REG_X, OP_REG_Y, 1,1, ST7_NO },
  // HALT
  { STM8_HALT, OP_NONE, 0, 0x8e, OP_NONE, OP_NONE, 1,1, ST7_YES },
  // INC
  // FIXME: the .w is outside the braces or non-existent.  Is this a typo
  // in the documentation?
  { STM8_INC, OP_SINGLE_REGISTER, 0, 0x4c, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_INC, OP_ADDRESS8, 0, 0x3c, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_INC, OP_ADDRESS16, 0x72, 0x5c, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_INC, OP_INDEX_X, 0, 0x7c, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_INC, OP_OFFSET8_INDEX_X, 0, 0x6c, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_INC, OP_OFFSET16_INDEX_X, 0x72, 0x4c, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_INC, OP_INDEX_Y, 0x90, 0x7c, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_INC, OP_OFFSET8_INDEX_Y, 0x90, 0x6c, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_INC, OP_OFFSET16_INDEX_Y, 0x90, 0x4c, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_INC, OP_OFFSET8_INDEX_SP, 0, 0x0c, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_INC, OP_INDIRECT8, 0x92, 0x3c, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_INC, OP_INDIRECT16, 0x72, 0x3c, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_INC, OP_INDIRECT8_X, 0x92, 0x6c, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_INC, OP_INDIRECT16_X, 0x72, 0x6c, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_INC, OP_INDIRECT8_Y, 0x91, 0x6c, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // INCW
  { STM8_INCW, OP_SINGLE_REGISTER, 0, 0x5c, OP_REG_X, OP_NONE, 1,1, ST7_NO },
  { STM8_INCW, OP_SINGLE_REGISTER, 0x90, 0x5c, OP_REG_Y, OP_NONE, 1,1, ST7_NO },
  // INT
  { STM8_INT, OP_ADDRESS24, 0, 0x82, OP_NONE, OP_NONE, 2,2, ST7_NO },
  // IRET
  { STM8_IRET, OP_NONE, 0, 0x80, OP_NONE, OP_NONE, 11,11, ST7_YES },
  // JP
  { STM8_JP, OP_ADDRESS16, 0, 0xcc, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_JP, OP_INDEX_X, 0, 0xfc, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_JP, OP_OFFSET8_INDEX_X, 0, 0xec, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_JP, OP_OFFSET16_INDEX_X, 0, 0xdc, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_JP, OP_INDEX_Y, 0x90, 0xfc, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_JP, OP_OFFSET8_INDEX_Y, 0x90, 0xec, OP_NONE, OP_NONE, 2,2, ST7_YES },
  { STM8_JP, OP_OFFSET16_INDEX_Y, 0x90, 0xdc, OP_NONE, OP_NONE, 2,2, ST7_YES },
  { STM8_JP, OP_INDIRECT8, 0x92, 0xcc, OP_NONE, OP_NONE, 5,5, ST7_YES },
  { STM8_JP, OP_INDIRECT16, 0x72, 0xcc, OP_NONE, OP_NONE, 5,5, ST7_NO },
  { STM8_JP, OP_INDIRECT8_X, 0x92, 0xdc, OP_NONE, OP_NONE, 5,5, ST7_YES },
  { STM8_JP, OP_INDIRECT16_X, 0x72, 0xdc, OP_NONE, OP_NONE, 5,5, ST7_NO },
  { STM8_JP, OP_INDIRECT8_Y, 0x91, 0xdc, OP_NONE, OP_NONE, 5,5, ST7_YES },
  // JPF
  { STM8_JPF, OP_ADDRESS24, 0, 0xac, OP_NONE, OP_NONE, 2,2, ST7_NO },
  { STM8_JPF, OP_INDIRECT16_E, 0x92, 0xac, OP_NONE, OP_NONE, 6,6, ST7_NO },
  // JRA
  { STM8_JRA, OP_RELATIVE, 0, 0x20, OP_NONE, OP_NONE, 2,2, ST7_YES },
  // JRxx
  { STM8_JRC, OP_RELATIVE, 0, 0x25, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JREQ, OP_RELATIVE, 0, 0x27, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRF, OP_RELATIVE, 0, 0x21, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRH, OP_RELATIVE, 0x90, 0x29, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRIH, OP_RELATIVE, 0x90, 0x2f, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRIL, OP_RELATIVE, 0x90, 0x2e, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRM, OP_RELATIVE, 0x90, 0x2d, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRMI, OP_RELATIVE, 0, 0x2b, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRNC, OP_RELATIVE, 0, 0x24, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRNE, OP_RELATIVE, 0, 0x26, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRNH, OP_RELATIVE, 0x90, 0x28, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRNM, OP_RELATIVE, 0x90, 0x2c, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRNV, OP_RELATIVE, 0, 0x28, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRPL, OP_RELATIVE, 0, 0x2a, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRSGE, OP_RELATIVE, 0, 0x2e, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRSGT, OP_RELATIVE, 0, 0x2c, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRSLE, OP_RELATIVE, 0, 0x2d, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRSLT, OP_RELATIVE, 0, 0x2f, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRT, OP_RELATIVE, 0, 0x20, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRUGE, OP_RELATIVE, 0, 0x24, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRUGT, OP_RELATIVE, 0, 0x22, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRULE, OP_RELATIVE, 0, 0x23, OP_NONE, OP_NONE, 1,2, ST7_YES },
  //{ STM8_JRC, OP_RELATIVE, 0, 0x25, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRULT, OP_RELATIVE, 0, 0x25, OP_NONE, OP_NONE, 1,2, ST7_YES },
  { STM8_JRV, OP_RELATIVE, 0, 0x29, OP_NONE, OP_NONE, 1,2, ST7_YES },
  // LD A, <xxx>
  { STM8_LD, OP_NUMBER8, 0, 0xa6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_ADDRESS8, 0, 0xb6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_ADDRESS16, 0, 0xc6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_INDEX_X, 0, 0xf6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET8_INDEX_X, 0, 0xe6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET16_INDEX_X, 0, 0xd6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_INDEX_Y, 0x90, 0xf6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET8_INDEX_Y, 0x90, 0xe6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET16_INDEX_Y, 0x90, 0xd6, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET8_INDEX_SP, 0, 0x7b, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_LD, OP_INDIRECT8, 0x92, 0xc6, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_LD, OP_INDIRECT16, 0x72, 0xc6, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_LD, OP_INDIRECT8_X, 0x92, 0xd6, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_LD, OP_INDIRECT16_X, 0x72, 0xd6, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_LD, OP_INDIRECT8_Y, 0x91, 0xd6, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  // LD <xxx>, A
  { STM8_LD, OP_ADDRESS8, 0, 0xb7, OP_NONE, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_ADDRESS16, 0, 0xc7, OP_NONE, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_INDEX_X, 0, 0xf7, OP_NONE, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET8_INDEX_X, 0, 0xe7, OP_NONE, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET16_INDEX_X, 0, 0xd7, OP_NONE, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_INDEX_Y, 0x90, 0xf7, OP_NONE, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET8_INDEX_Y, 0x90, 0xe7, OP_NONE, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET16_INDEX_Y, 0x90, 0xd7, OP_NONE, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_OFFSET8_INDEX_SP, 0, 0x6b, OP_NONE, OP_REG_A, 1,1, ST7_NO },
  { STM8_LD, OP_INDIRECT8, 0x92, 0xc7, OP_NONE, OP_REG_A, 4,4, ST7_YES },
  { STM8_LD, OP_INDIRECT16, 0x72, 0xc7, OP_NONE, OP_REG_A, 4,4, ST7_NO },
  { STM8_LD, OP_INDIRECT8_X, 0x92, 0xd7, OP_NONE, OP_REG_A, 4,4, ST7_YES },
  { STM8_LD, OP_INDIRECT16_X, 0x72, 0xd7, OP_NONE, OP_REG_A, 4,4, ST7_NO },
  { STM8_LD, OP_INDIRECT8_Y, 0x91, 0xd7, OP_NONE, OP_REG_A, 4,4, ST7_YES },
  // LD REG, REG
  { STM8_LD, OP_TWO_REGISTERS, 0, 0x97, OP_REG_XL, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_TWO_REGISTERS, 0, 0x9f, OP_REG_A, OP_REG_XL, 1,1, ST7_YES },
  { STM8_LD, OP_TWO_REGISTERS, 0x90, 0x97, OP_REG_YL, OP_REG_A, 1,1, ST7_YES },
  { STM8_LD, OP_TWO_REGISTERS, 0x90, 0x9f, OP_REG_A, OP_REG_YL, 1,1, ST7_YES },
  { STM8_LD, OP_TWO_REGISTERS, 0, 0x95, OP_REG_XH, OP_REG_A, 1,1, ST7_NO },
  { STM8_LD, OP_TWO_REGISTERS, 0, 0x9e, OP_REG_A, OP_REG_XH, 1,1, ST7_NO },
  { STM8_LD, OP_TWO_REGISTERS, 0x90, 0x95, OP_REG_YH, OP_REG_A, 1,1, ST7_NO },
  { STM8_LD, OP_TWO_REGISTERS, 0x90, 0x9e, OP_REG_A, OP_REG_YH, 1,1, ST7_NO },
  // LDF A, <xxx>
  { STM8_LDF, OP_ADDRESS24, 0, 0xbc, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_LDF, OP_OFFSET24_INDEX_X, 0, 0xaf, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_LDF, OP_OFFSET24_INDEX_Y, 0x90, 0xaf, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_LDF, OP_INDIRECT16_E_X, 0x92, 0xaf, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_LDF, OP_INDIRECT16_E_Y, 0x91, 0xaf, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_LDF, OP_INDIRECT16_E, 0x92, 0xbc, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  // LDF <xxx>, A
  { STM8_LDF, OP_ADDRESS24, 0, 0xbd, OP_NONE, OP_REG_A, 1,1, ST7_NO },
  { STM8_LDF, OP_OFFSET24_INDEX_X, 0, 0xa7, OP_NONE, OP_REG_A, 1,1, ST7_NO },
  { STM8_LDF, OP_OFFSET24_INDEX_Y, 0x90, 0xa7, OP_NONE, OP_REG_A, 1,1, ST7_NO },
  { STM8_LDF, OP_INDIRECT16_E_X, 0x92, 0xa7, OP_NONE, OP_REG_A, 4,4, ST7_NO },
  { STM8_LDF, OP_INDIRECT16_E_Y, 0x91, 0xa7, OP_NONE, OP_REG_A, 4,4, ST7_NO },
  { STM8_LDF, OP_INDIRECT16_E, 0x92, 0xbd, OP_NONE, OP_REG_A, 4,4, ST7_NO },
  // LDW X, <xxx>
  // FIXME: Check if this is really INDEX_X.  This contradicts previous
  // instructions.  Also, it has an extra instruction.
  { STM8_LDW, OP_NUMBER16, 0, 0xae, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_ADDRESS8, 0, 0xbe, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_ADDRESS16, 0, 0xce, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_INDEX_X, 0, 0xfe, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET8_INDEX_X, 0, 0xee, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET16_INDEX_X, 0, 0xde, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET8_INDEX_SP, 0, 0x1e, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_LDW, OP_INDIRECT8, 0x92, 0xce, OP_REG_X, OP_NONE, 5,5, ST7_YES },
  { STM8_LDW, OP_INDIRECT16, 0x72, 0xce, OP_REG_X, OP_NONE, 5,5, ST7_NO },
  { STM8_LDW, OP_INDIRECT8_X, 0x92, 0xde, OP_REG_X, OP_NONE, 5,5, ST7_YES },
  { STM8_LDW, OP_INDIRECT16_X, 0x72, 0xde, OP_REG_X, OP_NONE, 5,5, ST7_NO },

  { STM8_LDW, OP_ADDRESS8, 0, 0xbf, OP_NONE, OP_REG_X, 2,2, ST7_YES },
  { STM8_LDW, OP_ADDRESS16, 0, 0xcf, OP_NONE, OP_REG_X, 2,2, ST7_YES },
  { STM8_LDW, OP_INDEX_X, 0, 0xff, OP_NONE, OP_REG_Y, 2,2, ST7_NO },
  { STM8_LDW, OP_OFFSET8_INDEX_X, 0, 0xef, OP_NONE, OP_REG_Y, 2,2, ST7_NO },
  { STM8_LDW, OP_OFFSET16_INDEX_X, 0, 0xdf, OP_NONE, OP_REG_Y, 2,2, ST7_NO },
  { STM8_LDW, OP_OFFSET8_INDEX_SP, 0, 0x1f, OP_NONE, OP_REG_X, 2,2, ST7_NO },
  { STM8_LDW, OP_INDIRECT8, 0x92, 0xcf, OP_NONE, OP_REG_X, 5,5, ST7_YES },
  { STM8_LDW, OP_INDIRECT16, 0x72, 0xcf, OP_NONE, OP_REG_X, 5,5, ST7_NO },
  { STM8_LDW, OP_INDIRECT8_X, 0x92, 0xdf, OP_NONE, OP_REG_Y, 5,5, ST7_YES },
  { STM8_LDW, OP_INDIRECT16_X, 0x72, 0xdf, OP_NONE, OP_REG_Y, 5,5, ST7_NO },

  { STM8_LDW, OP_NUMBER16, 0x90, 0xae, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_ADDRESS8, 0x90, 0xbe, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_ADDRESS16, 0x90, 0xce, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_INDEX_Y, 0x90, 0xfe, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET8_INDEX_Y, 0x90, 0xee, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET16_INDEX_Y, 0x90, 0xde, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET8_INDEX_SP, 0, 0x16, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  { STM8_LDW, OP_INDIRECT8, 0x91, 0xce, OP_REG_Y, OP_NONE, 5,5, ST7_YES },
  { STM8_LDW, OP_INDIRECT8_Y, 0x91, 0xde, OP_REG_Y, OP_NONE, 5,5, ST7_YES },

  { STM8_LDW, OP_ADDRESS8, 0x90, 0xbf, OP_NONE, OP_REG_Y, 2,2, ST7_YES },
  { STM8_LDW, OP_ADDRESS16, 0x90, 0xcf, OP_NONE, OP_REG_Y, 2,2, ST7_YES },
  { STM8_LDW, OP_INDEX_Y, 0x90, 0xff, OP_NONE, OP_REG_X, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET8_INDEX_Y, 0x90, 0xef, OP_NONE, OP_REG_X, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET16_INDEX_Y, 0x90, 0xdf, OP_NONE, OP_REG_X, 2,2, ST7_YES },
  { STM8_LDW, OP_OFFSET8_INDEX_SP, 0, 0x17, OP_NONE, OP_REG_Y, 2,2, ST7_NO },
  { STM8_LDW, OP_INDIRECT8, 0x91, 0xcf, OP_NONE, OP_REG_Y, 5,5, ST7_YES },
  { STM8_LDW, OP_INDIRECT8_Y, 0x91, 0xdf, OP_NONE, OP_REG_X, 5,5, ST7_YES },

  { STM8_LDW, OP_TWO_REGISTERS, 0x90, 0x93, OP_REG_Y, OP_REG_X, 1,1, ST7_YES },
  { STM8_LDW, OP_TWO_REGISTERS, 0, 0x93, OP_REG_X, OP_REG_Y, 1,1, ST7_YES },
  { STM8_LDW, OP_TWO_REGISTERS, 0, 0x96, OP_REG_X, OP_SP, 1,1, ST7_YES },
  { STM8_LDW, OP_TWO_REGISTERS, 0, 0x94, OP_SP, OP_REG_X, 1,1, ST7_YES },
  { STM8_LDW, OP_TWO_REGISTERS, 0x90, 0x96, OP_REG_Y, OP_SP, 1,1, ST7_YES },
  { STM8_LDW, OP_TWO_REGISTERS, 0x90, 0x94, OP_SP, OP_REG_Y, 1,1, ST7_YES },
  // MOV
  { STM8_MOV, OP_ADDRESS16_NUMBER8, 0, 0x35, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_MOV, OP_ADDRESS8_ADDRESS8, 0, 0x45, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_MOV, OP_ADDRESS16_ADDRESS16, 0, 0x55, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // MUL
  { STM8_MUL, OP_TWO_REGISTERS, 0, 0x42, OP_REG_X, OP_REG_A, 4,4, ST7_NO },
  { STM8_MUL, OP_TWO_REGISTERS, 0x90, 0x42, OP_REG_Y, OP_REG_A, 4,4, ST7_NO },
  // NEG
  { STM8_NEG, OP_SINGLE_REGISTER, 0, 0x40, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_NEG, OP_ADDRESS8, 0, 0x30, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_NEG, OP_ADDRESS16, 0x72, 0x50, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_NEG, OP_INDEX_X, 0, 0x70, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_NEG, OP_OFFSET8_INDEX_X, 0, 0x60, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_NEG, OP_OFFSET16_INDEX_X, 0x72, 0x40, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_NEG, OP_INDEX_Y, 0x90, 0x70, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_NEG, OP_OFFSET8_INDEX_Y, 0x90, 0x60, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_NEG, OP_OFFSET16_INDEX_Y, 0x90, 0x40, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_NEG, OP_OFFSET8_INDEX_SP, 0, 0x00, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_NEG, OP_INDIRECT8, 0x92, 0x30, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_NEG, OP_INDIRECT16, 0x72, 0x30, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_NEG, OP_INDIRECT8_X, 0x92, 0x60, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_NEG, OP_INDIRECT16_X, 0x72, 0x60, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_NEG, OP_INDIRECT8_Y, 0x91, 0x60, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // NEGW
  { STM8_NEGW, OP_SINGLE_REGISTER, 0, 0x50, OP_REG_X, OP_NONE, 1,1, ST7_NO },
  { STM8_NEGW, OP_SINGLE_REGISTER, 0x90, 0x50, OP_REG_Y, OP_NONE, 1,1, ST7_NO },
  // NOP
  { STM8_NOP, OP_NONE, 0, 0x9d, OP_NONE, OP_NONE, 1,1, ST7_YES },
  // OR
  { STM8_OR, OP_NUMBER8, 0, 0xaa, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_ADDRESS8, 0, 0xba, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_ADDRESS16, 0, 0xca, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_INDEX_X, 0, 0xfa, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_OFFSET8_INDEX_X, 0, 0xea, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_OFFSET16_INDEX_X, 0, 0xda, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_INDEX_Y, 0x90, 0xfa, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_OFFSET8_INDEX_Y, 0x90, 0xea, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_OFFSET16_INDEX_Y, 0x90, 0xda, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_OR, OP_OFFSET8_INDEX_SP, 0, 0x1a, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_OR, OP_INDIRECT8, 0x92, 0xca, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_OR, OP_INDIRECT16, 0x72, 0xca, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_OR, OP_INDIRECT8_X, 0x92, 0xda, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_OR, OP_INDIRECT16_X, 0x72, 0xda, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_OR, OP_INDIRECT8_Y, 0x91, 0xda, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  // POP
  { STM8_POP, OP_SINGLE_REGISTER, 0, 0x84, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_POP, OP_SINGLE_REGISTER, 0, 0x86, OP_REG_CC, OP_NONE, 1,1, ST7_YES },
  { STM8_POP, OP_ADDRESS16, 0, 0x32, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // POPW
  // FIXME: WTF here?  all the W instructions don't have ST7 marked, but in
  // the stm8.pdf it is marked.  This looks like another mistake.
  { STM8_POPW, OP_SINGLE_REGISTER, 0, 0x85, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_POPW, OP_SINGLE_REGISTER, 0x90, 0x85, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
   // PUSH
  { STM8_PUSH, OP_SINGLE_REGISTER, 0, 0x88, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_PUSH, OP_SINGLE_REGISTER, 0, 0x8a, OP_REG_CC, OP_NONE, 1,1, ST7_YES },
  { STM8_PUSH, OP_NUMBER8, 0, 0x4b, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_PUSH, OP_ADDRESS16, 0, 0x3b, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // PUSHW
  // FIXME: Again it shows ST7.
  { STM8_PUSHW, OP_SINGLE_REGISTER, 0, 0x89, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_PUSHW, OP_SINGLE_REGISTER, 0x90, 0x89, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  // RCF
  { STM8_RCF, OP_NONE, 0, 0x98, OP_NONE, OP_NONE, 1,1, ST7_YES },
  // RET
  { STM8_RET, OP_NONE, 0, 0x81, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // RETF
  { STM8_RETF, OP_NONE, 0, 0x87, OP_NONE, OP_NONE, 5,5, ST7_NO },
  // RIM
  { STM8_RIM, OP_NONE, 0, 0x9a, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // RLC
  // FIXME: No other instruction has ($10,SP) support for ST7... why is it
  // set here?  This again looks like a documentation error.
  { STM8_RLC, OP_SINGLE_REGISTER, 0, 0x49, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_RLC, OP_ADDRESS8, 0, 0x39, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RLC, OP_ADDRESS16, 0x72, 0x59, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_RLC, OP_INDEX_X, 0, 0x79, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RLC, OP_OFFSET8_INDEX_X, 0, 0x69, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RLC, OP_OFFSET16_INDEX_X, 0x72, 0x49, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_RLC, OP_INDEX_Y, 0x90, 0x79, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RLC, OP_OFFSET8_INDEX_Y, 0x90, 0x69, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RLC, OP_OFFSET16_INDEX_Y, 0x90, 0x49, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_RLC, OP_OFFSET8_INDEX_SP, 0, 0x09, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RLC, OP_INDIRECT8, 0x92, 0x39, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_RLC, OP_INDIRECT16, 0x72, 0x39, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_RLC, OP_INDIRECT8_X, 0x92, 0x69, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_RLC, OP_INDIRECT16_X, 0x72, 0x69, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_RLC, OP_INDIRECT8_Y, 0x91, 0x69, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // RLCW
  // FIXME: Again it shows ST7.
  { STM8_RLCW, OP_SINGLE_REGISTER, 0, 0x59, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_RLCW, OP_SINGLE_REGISTER, 0x90, 0x59, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  // RLWA
  { STM8_RLWA, OP_TWO_REGISTERS, 0, 0x02, OP_REG_X, OP_REG_A, 1,1, ST7_NO },
  { STM8_RLWA, OP_TWO_REGISTERS, 0x90, 0x02, OP_REG_Y, OP_REG_A, 1,1, ST7_NO },
  // RRC
  // FIXME: No other instruction has ($10,SP) support for ST7... why is it
  // set here?  This again looks like a documentation error.
  { STM8_RRC, OP_SINGLE_REGISTER, 0, 0x46, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_RRC, OP_ADDRESS8, 0, 0x36, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RRC, OP_ADDRESS16, 0x72, 0x56, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_RRC, OP_INDEX_X, 0, 0x76, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RRC, OP_OFFSET8_INDEX_X, 0, 0x66, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RRC, OP_OFFSET16_INDEX_X, 0x72, 0x46, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_RRC, OP_INDEX_Y, 0x90, 0x76, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RRC, OP_OFFSET8_INDEX_Y, 0x90, 0x66, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RRC, OP_OFFSET16_INDEX_Y, 0x90, 0x46, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_RRC, OP_OFFSET8_INDEX_SP, 0, 0x06, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_RRC, OP_INDIRECT8, 0x92, 0x36, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_RRC, OP_INDIRECT16, 0x72, 0x36, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_RRC, OP_INDIRECT8_X, 0x92, 0x66, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_RRC, OP_INDIRECT16_X, 0x72, 0x66, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_RRC, OP_INDIRECT8_Y, 0x91, 0x66, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // RRCW
  // FIXME: Again it shows ST7.
  { STM8_RRCW, OP_SINGLE_REGISTER, 0, 0x56, OP_REG_X, OP_NONE, 2,2, ST7_YES },
  { STM8_RRCW, OP_SINGLE_REGISTER, 0x90, 0x56, OP_REG_Y, OP_NONE, 2,2, ST7_YES },
  // RRWA
  { STM8_RRWA, OP_TWO_REGISTERS, 0, 0x01, OP_REG_X, OP_REG_A, 1,1, ST7_NO },
  { STM8_RRWA, OP_TWO_REGISTERS, 0x90, 0x01, OP_REG_Y, OP_REG_A, 1,1, ST7_NO },
  // RVF
  { STM8_RVF, OP_NONE, 0, 0x9c, OP_NONE, OP_NONE, 5,5, ST7_YES },
  // SBC
  { STM8_SBC, OP_NUMBER8, 0, 0xa2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_ADDRESS8, 0, 0xb2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_ADDRESS16, 0, 0xc2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_INDEX_X, 0, 0xf2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_OFFSET8_INDEX_X, 0, 0xe2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_OFFSET16_INDEX_X, 0, 0xd2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_INDEX_Y, 0x90, 0xf2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_OFFSET8_INDEX_Y, 0x90, 0xe2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_OFFSET16_INDEX_Y, 0x90, 0xd2, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SBC, OP_OFFSET8_INDEX_SP, 0, 0x12, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_SBC, OP_INDIRECT8, 0x92, 0xc2, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_SBC, OP_INDIRECT16, 0x72, 0xc2, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_SBC, OP_INDIRECT8_X, 0x92, 0xd2, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_SBC, OP_INDIRECT16_X, 0x72, 0xd2, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_SBC, OP_INDIRECT8_Y, 0x91, 0xd2, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  // SCF
  { STM8_SCF, OP_NONE, 0, 0x99, OP_NONE, OP_NONE, 1,1, ST7_YES },
  // SIM
  { STM8_SIM, OP_NONE, 0, 0x9b, OP_NONE, OP_NONE, 1,1, ST7_YES },
  // SLL
  // FIXME: No other instruction has ($10,SP) support for ST7... why is it
  // set here?  This again looks like a documentation error.
  { STM8_SLL, OP_SINGLE_REGISTER, 0, 0x48, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SLL, OP_ADDRESS8, 0, 0x38, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SLL, OP_ADDRESS16, 0x72, 0x58, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SLL, OP_INDEX_X, 0, 0x78, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SLL, OP_OFFSET8_INDEX_X, 0, 0x68, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SLL, OP_OFFSET16_INDEX_X, 0x72, 0x48, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SLL, OP_INDEX_Y, 0x90, 0x78, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SLL, OP_OFFSET8_INDEX_Y, 0x90, 0x68, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SLL, OP_OFFSET16_INDEX_Y, 0x90, 0x48, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SLL, OP_OFFSET8_INDEX_SP, 0, 0x08, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SLL, OP_INDIRECT8, 0x92, 0x38, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_SLL, OP_INDIRECT16, 0x72, 0x38, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_SLL, OP_INDIRECT8_X, 0x92, 0x68, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_SLL, OP_INDIRECT16_X, 0x72, 0x68, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_SLL, OP_INDIRECT8_Y, 0x91, 0x68, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // SLA
  // Instead of making this table bigger, going to take care of an alias
  // in asm_stm8.c towards the top.
  // SLLW
  { STM8_SLLW, OP_SINGLE_REGISTER, 0, 0x58, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_SLLW, OP_SINGLE_REGISTER, 0x90, 0x58, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  // SLAW
  // Instead of making this table bigger, going to take care of an alias
  // in asm_stm8.c towards the top.
  // SRA
  // FIXME: Again ($10,SP) is marked as ST7.
  { STM8_SRA, OP_SINGLE_REGISTER, 0, 0x47, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SRA, OP_ADDRESS8, 0, 0x37, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRA, OP_ADDRESS16, 0x72, 0x57, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SRA, OP_INDEX_X, 0, 0x77, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRA, OP_OFFSET8_INDEX_X, 0, 0x67, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRA, OP_OFFSET16_INDEX_X, 0x72, 0x47, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SRA, OP_INDEX_Y, 0x90, 0x77, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRA, OP_OFFSET8_INDEX_Y, 0x90, 0x67, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRA, OP_OFFSET16_INDEX_Y, 0x90, 0x47, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SRA, OP_OFFSET8_INDEX_SP, 0, 0x07, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRA, OP_INDIRECT8, 0x92, 0x37, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_SRA, OP_INDIRECT16, 0x72, 0x37, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_SRA, OP_INDIRECT8_X, 0x92, 0x67, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_SRA, OP_INDIRECT16_X, 0x72, 0x67, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_SRA, OP_INDIRECT8_Y, 0x91, 0x67, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // SRAW
  { STM8_SRAW, OP_SINGLE_REGISTER, 0, 0x57, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_SRAW, OP_SINGLE_REGISTER, 0x90, 0x57, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  // SRL
  // FIXME: Again ($10,SP) is marked as ST7.
  { STM8_SRL, OP_SINGLE_REGISTER, 0, 0x44, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SRL, OP_ADDRESS8, 0, 0x34, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRL, OP_ADDRESS16, 0x72, 0x54, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SRL, OP_INDEX_X, 0, 0x74, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRL, OP_OFFSET8_INDEX_X, 0, 0x64, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRL, OP_OFFSET16_INDEX_X, 0x72, 0x44, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SRL, OP_INDEX_Y, 0x90, 0x74, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRL, OP_OFFSET8_INDEX_Y, 0x90, 0x64, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRL, OP_OFFSET16_INDEX_Y, 0x90, 0x44, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SRL, OP_OFFSET8_INDEX_SP, 0, 0x04, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SRL, OP_INDIRECT8, 0x92, 0x34, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_SRL, OP_INDIRECT16, 0x72, 0x34, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_SRL, OP_INDIRECT8_X, 0x92, 0x64, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_SRL, OP_INDIRECT16_X, 0x72, 0x64, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_SRL, OP_INDIRECT8_Y, 0x91, 0x64, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // SRLW
  { STM8_SRLW, OP_SINGLE_REGISTER, 0, 0x54, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_SRLW, OP_SINGLE_REGISTER, 0x90, 0x54, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  // SUB
  { STM8_SUB, OP_NUMBER8, 0, 0xa0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_ADDRESS8, 0, 0xb0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_ADDRESS16, 0, 0xc0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_INDEX_X, 0, 0xf0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_OFFSET8_INDEX_X, 0, 0xe0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_OFFSET16_INDEX_X, 0, 0xd0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_INDEX_Y, 0x90, 0xf0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_OFFSET8_INDEX_Y, 0x90, 0xe0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_OFFSET16_INDEX_Y, 0x90, 0xd0, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SUB, OP_OFFSET8_INDEX_SP, 0, 0x10, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_SUB, OP_INDIRECT8, 0x92, 0xc0, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_SUB, OP_INDIRECT16, 0x72, 0xc0, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_SUB, OP_INDIRECT8_X, 0x92, 0xd0, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_SUB, OP_INDIRECT16_X, 0x72, 0xd0, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_SUB, OP_INDIRECT8_Y, 0x91, 0xd0, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_SUB, OP_NUMBER8, 0, 0x52, OP_SP, OP_NONE, 1,1, ST7_NO },
  // SUBW
  { STM8_SUBW, OP_NUMBER16, 0, 0x1d, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_SUBW, OP_ADDRESS16, 0x72, 0xb0, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_SUBW, OP_OFFSET8_INDEX_SP, 0x72, 0xf0, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_SUBW, OP_NUMBER16, 0x72, 0xa2, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  { STM8_SUBW, OP_ADDRESS16, 0x72, 0xb2, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  { STM8_SUBW, OP_OFFSET8_INDEX_SP, 0x72, 0xf2, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  { STM8_SUBW, OP_NUMBER8, 0, 0x52, OP_SP, OP_NONE, 2,2, ST7_NO },
  // SWAP
  // FIXME: Again ($10,SP) is marked as ST7.
  { STM8_SWAP, OP_SINGLE_REGISTER, 0, 0x4e, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_SWAP, OP_ADDRESS8, 0, 0x3e, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SWAP, OP_ADDRESS16, 0x72, 0x5e, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SWAP, OP_INDEX_X, 0, 0x7e, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SWAP, OP_OFFSET8_INDEX_X, 0, 0x6e, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SWAP, OP_OFFSET16_INDEX_X, 0x72, 0x4e, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SWAP, OP_INDEX_Y, 0x90, 0x7e, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SWAP, OP_OFFSET8_INDEX_Y, 0x90, 0x6e, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SWAP, OP_OFFSET16_INDEX_Y, 0x90, 0x4e, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_SWAP, OP_OFFSET8_INDEX_SP, 0, 0x0e, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_SWAP, OP_INDIRECT8, 0x92, 0x3e, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_SWAP, OP_INDIRECT16, 0x72, 0x3e, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_SWAP, OP_INDIRECT8_X, 0x92, 0x6e, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_SWAP, OP_INDIRECT16_X, 0x72, 0x6e, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_SWAP, OP_INDIRECT8_Y, 0x91, 0x6e, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // SWAPW
  { STM8_SWAPW, OP_SINGLE_REGISTER, 0, 0x5e, OP_REG_X, OP_NONE, 1,1, ST7_YES },
  { STM8_SWAPW, OP_SINGLE_REGISTER, 0x90, 0x5e, OP_REG_Y, OP_NONE, 1,1, ST7_YES },
  // TNZ
  // FIXME: Again ($10,SP) is marked as ST7.
  { STM8_TNZ, OP_SINGLE_REGISTER, 0, 0x4d, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_TNZ, OP_ADDRESS8, 0, 0x3d, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_TNZ, OP_ADDRESS16, 0x72, 0x5d, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_TNZ, OP_INDEX_X, 0, 0x7d, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_TNZ, OP_OFFSET8_INDEX_X, 0, 0x6d, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_TNZ, OP_OFFSET16_INDEX_X, 0x72, 0x4d, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_TNZ, OP_INDEX_Y, 0x90, 0x7d, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_TNZ, OP_OFFSET8_INDEX_Y, 0x90, 0x6d, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_TNZ, OP_OFFSET16_INDEX_Y, 0x90, 0x4d, OP_NONE, OP_NONE, 1,1, ST7_NO },
  { STM8_TNZ, OP_OFFSET8_INDEX_SP, 0, 0x0d, OP_NONE, OP_NONE, 1,1, ST7_YES },
  { STM8_TNZ, OP_INDIRECT8, 0x92, 0x3d, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_TNZ, OP_INDIRECT16, 0x72, 0x3d, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_TNZ, OP_INDIRECT8_X, 0x92, 0x6d, OP_NONE, OP_NONE, 4,4, ST7_YES },
  { STM8_TNZ, OP_INDIRECT16_X, 0x72, 0x6d, OP_NONE, OP_NONE, 4,4, ST7_NO },
  { STM8_TNZ, OP_INDIRECT8_Y, 0x91, 0x6d, OP_NONE, OP_NONE, 4,4, ST7_YES },
  // TNZW
  { STM8_TNZW, OP_SINGLE_REGISTER, 0, 0x5d, OP_REG_X, OP_NONE, 2,2, ST7_NO },
  { STM8_TNZW, OP_SINGLE_REGISTER, 0x90, 0x5d, OP_REG_Y, OP_NONE, 2,2, ST7_NO },
  // TRAP
  { STM8_TRAP, OP_NONE, 0, 0x83, OP_NONE, OP_NONE, 9,9, ST7_YES },
  // WFE
  { STM8_WFE, OP_NONE, 0x72, 0x8f, OP_NONE, OP_NONE, 1,1, ST7_NO },
  // WFI
  { STM8_WFI, OP_NONE, 0, 0x8f, OP_NONE, OP_NONE, 10,10, ST7_YES },
  // XOR
  { STM8_XOR, OP_NUMBER8, 0, 0xa8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_ADDRESS8, 0, 0xb8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_ADDRESS16, 0, 0xc8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_INDEX_X, 0, 0xf8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_OFFSET8_INDEX_X, 0, 0xe8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_OFFSET16_INDEX_X, 0, 0xd8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_INDEX_Y, 0x90, 0xf8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_OFFSET8_INDEX_Y, 0x90, 0xe8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_OFFSET16_INDEX_Y, 0x90, 0xd8, OP_REG_A, OP_NONE, 1,1, ST7_YES },
  { STM8_XOR, OP_OFFSET8_INDEX_SP, 0, 0x18, OP_REG_A, OP_NONE, 1,1, ST7_NO },
  { STM8_XOR, OP_INDIRECT8, 0x92, 0xc8, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_XOR, OP_INDIRECT16, 0x72, 0xc8, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_XOR, OP_INDIRECT8_X, 0x92, 0xd8, OP_REG_A, OP_NONE, 4,4, ST7_YES },
  { STM8_XOR, OP_INDIRECT16_X, 0x72, 0xd8, OP_REG_A, OP_NONE, 4,4, ST7_NO },
  { STM8_XOR, OP_INDIRECT8_Y, 0x91, 0xd8, OP_REG_A, OP_NONE, 4,4, ST7_YES },

  { STM8_NONE, 0, 0, 0, 0, 0, 0, 0 },
};

